from __future__ import annotations
from typing import Any

from checkov.common.util.consts import START_LINE, END_LINE

IMAGE_BLOCK_NAMES = ('image', 'services')


def generate_resource_key_recursive(conf: dict[str, Any] | list[str] | str, key: str, start_line: int,
                                    end_line: int) -> str:
    return _generate_resource_key_recursive(conf, key, start_line, end_line, set(), 0)


def _generate_resource_key_recursive(conf: dict[str, Any] | list[str] | str, key: str, start_line: int,
                                     end_line: int, scanned_image_blocks: set[str], depth: int) -> str:
    if not isinstance(conf, dict):
        return key

    for k, value in conf.items():
        if k in IMAGE_BLOCK_NAMES:
            scanned_image_blocks.add(k)

        if isinstance(value, dict) and value[START_LINE] <= start_line <= end_line <= value[END_LINE]:
            next_key = f'{key}.{k}' if key else k
            return _generate_resource_key_recursive(value, next_key, start_line, end_line, scanned_image_blocks,
                                                    depth + 1)

        if isinstance(value, list) and isinstance(value[0], dict):
            next_key = f'{key}.{k}' if key else k

            for idx, value_dict in enumerate(value):
                if value_dict[START_LINE] <= start_line <= end_line <= value_dict[END_LINE]:
                    next_key += f'.{idx + 1}'
                    break  # There can be only one match in terms of line range

            return _generate_resource_key_recursive(value, next_key, start_line, end_line, scanned_image_blocks,
                                                    depth + 1)

        if any(block_name in conf.keys()
               and block_name not in scanned_image_blocks
               and isinstance(conf[block_name], dict)
               and conf[block_name].get(START_LINE) <= start_line
               and conf[block_name].get(END_LINE) >= end_line
               for block_name in IMAGE_BLOCK_NAMES):
            # Avoid settling for a too general resource id, when there are blocks which usually contain image names,
            # and that these blocks were not scanned yet & match the line range.
            continue

        if depth == 0:
            # Indicates the first recursive call. a heuristic that top-level entities should usually be disregarded
            # in case they are not dictionaries.
            continue

        if isinstance(value, list):
            if key:
                return f'{key}.{k}'
            else:
                continue
        if isinstance(value, str):
            return key

    return key



from __future__ import annotations

from typing import Any


from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.org_members import schema as org_members
from checkov.json_doc.enums import BlockType

MAX_ADMIN_COUNT = 3


class GithubMinimumAdminsInOrganization(BaseGithubCheck):
    def __init__(self) -> None:
        name = "Ensure minimum admins are set for the organization"
        id = "CKV_GITHUB_26"
        categories = (CheckCategories.SUPPLY_CHAIN, )
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=["*"],
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:  # type:ignore[override]
        ckv_metadata, conf = self.resolve_ckv_metadata_conf(conf=conf)
        if 'org_admins' in ckv_metadata.get('file_name', ''):
            if org_members.validate(conf):
                if len(conf) <= MAX_ADMIN_COUNT:
                    return CheckResult.PASSED
                else:
                    return CheckResult.FAILED
        return CheckResult.UNKNOWN


check = GithubMinimumAdminsInOrganization()



from typing import Dict, Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AppGatewayWAFACLCVE202144228(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Application Gateway WAF prevents message lookup in Log4j2. See CVE-2021-44228 aka log4jshell"
        id = "CKV_AZURE_135"
        supported_resources = ("azurerm_web_application_firewall_policy",)
        categories = (CheckCategories.APPLICATION_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_keys = ["managed_rules"]
        managed_rules = conf.get("managed_rules")
        if managed_rules:
            managed_rule_sets = managed_rules[0].get("managed_rule_set") or []
            for idx_rule_set, rule_set in enumerate(force_list(managed_rule_sets)):
                self.evaluated_keys = [
                    f"managed_rules/[0]/managed_rule_set[{idx_rule_set}]/type",
                    f"managed_rules/[0]/managed_rule_set[{idx_rule_set}]/version",
                ]
                if rule_set.get("type", ["OWASP"]) == ["OWASP"] and rule_set.get("version") in (["3.1"], ["3.2"]):
                    rule_overrides = rule_set.get("rule_group_override") or []
                    for idx_override, rule_override in enumerate(force_list(rule_overrides)):
                        self.evaluated_keys.extend(
                            [
                                f"managed_rules/[0]/managed_rule_set[{idx_rule_set}]/rule_group_override/[{idx_override}]/rule_group_name",
                                f"managed_rules/[0]/managed_rule_set[{idx_rule_set}]/rule_group_override/[{idx_override}]/disabled_rules",
                            ]
                        )
                        if isinstance(rule_override, dict) and rule_override.get("rule_group_name") == ["REQUEST-944-APPLICATION-ATTACK-JAVA"]:
                            disabled_rules = rule_override.get("disabled_rules") or []
                            if disabled_rules and isinstance(disabled_rules, list) and "944240" in force_list(disabled_rules[0]):
                                return CheckResult.FAILED

                    return CheckResult.PASSED

        return CheckResult.FAILED


check = AppGatewayWAFACLCVE202144228()



from __future__ import annotations

import logging
import os
from typing import Optional, List, Tuple, Dict, Any

from termcolor import colored

from checkov.common.models.enums import CheckResult
from checkov.secrets.consts import ValidationStatus

from checkov.common.bridgecrew.severities import Severity

from checkov.common.output.record import Record
from checkov.secrets.consts import GIT_HISTORY_NOT_BEEN_REMOVED
from checkov.common.typing import _CheckResult

COMMIT_ADDED_STR = 'Commit Added'
COMMIT_REMOVED_STR = 'Commit Removed'

WARNING_SIGN_UNICODE = '\u26a0'
TEXT_BY_SECRET_VALIDATION_STATUS = {
    ValidationStatus.VALID.value: colored(f'\t{WARNING_SIGN_UNICODE} This secret has been validated'
                                          f' and should be prioritized', "red"),
    ValidationStatus.INVALID.value: '\tThis is not a valid secret and can be de-prioritized',
    ValidationStatus.UNKNOWN.value: '\tWe were not able to validate this secret',
    ValidationStatus.UNAVAILABLE.value: ''
}


class SecretsRecord(Record):
    def __init__(self,
                 check_id: str,
                 check_name: str,
                 check_result: _CheckResult,
                 code_block: List[Tuple[int, str]],
                 file_path: str,
                 file_line_range: List[int],
                 resource: str,
                 evaluations: Optional[Dict[str, Any]],
                 check_class: str,
                 file_abs_path: str,
                 entity_tags: Optional[Dict[str, str]] = None,
                 caller_file_path: Optional[str] = None,
                 caller_file_line_range: Optional[Tuple[int, int]] = None,
                 bc_check_id: Optional[str] = None,
                 resource_address: Optional[str] = None,
                 severity: Optional[Severity] = None,
                 bc_category: Optional[str] = None,
                 benchmarks: dict[str, list[str]] | None = None,
                 description: Optional[str] = None,
                 short_description: Optional[str] = None,
                 vulnerability_details: Optional[Dict[str, Any]] = None,
                 connected_node: Optional[Dict[str, Any]] = None,
                 details: Optional[List[str]] = None,
                 check_len: int | None = None,
                 definition_context_file_path: Optional[str] = None,
                 validation_status: Optional[str] = None,
                 added_commit_hash: Optional[str] = None,
                 removed_commit_hash: Optional[str] = None
                 ):
        super().__init__(check_id=check_id,
                         check_name=check_name,
                         check_result=check_result,
                         code_block=code_block,
                         file_path=file_path,
                         file_line_range=file_line_range,
                         resource=resource,
                         evaluations=evaluations,
                         check_class=check_class,
                         file_abs_path=file_abs_path,
                         entity_tags=entity_tags,
                         bc_check_id=bc_check_id,
                         severity=severity,
                         details=details,
                         caller_file_path=caller_file_path,
                         caller_file_line_range=caller_file_line_range,
                         resource_address=resource_address,
                         bc_category=bc_category,
                         benchmarks=benchmarks,
                         description=description,
                         short_description=short_description,
                         vulnerability_details=vulnerability_details,
                         connected_node=connected_node,
                         check_len=check_len,
                         definition_context_file_path=definition_context_file_path
                         )
        self.validation_status = validation_status
        self.added_commit_hash = added_commit_hash
        self.removed_commit_hash = removed_commit_hash

    def to_string(self, compact: bool = False, use_bc_ids: bool = False) -> str:
        processed_record = super().to_string(compact=compact, use_bc_ids=use_bc_ids)
        validation_status_message = self._get_secret_validation_status_message()
        if validation_status_message and self.check_result["result"] == CheckResult.FAILED and os.getenv("CKV_VALIDATE_SECRETS"):
            # if needed insert validation status message
            splitted_record = processed_record.split("\n")
            splitted_record.insert(2, validation_status_message)
            processed_record = "\n".join(splitted_record)

        processed_record = self._add_commit_details(processed_record)
        return processed_record

    def _add_commit_details(self, processed_record: str) -> str:
        if not self.added_commit_hash and not self.is_empty_removed_commit():
            return processed_record
        splitted_record = processed_record.split("\n")
        file_idx = 0
        file_line = ''
        for idx, line in enumerate(splitted_record):
            if line.__contains__('File:'):
                file_idx = idx
                file_line = line
                break
        added = False
        if self.added_commit_hash:
            file_line = file_line + f'; {COMMIT_ADDED_STR}: {self.added_commit_hash}'
            added = True
        if self.removed_commit_hash:
            file_line = file_line + f'; {COMMIT_REMOVED_STR}: {self.removed_commit_hash}'
            added = True
        if added:
            splitted_record[file_idx] = file_line
            processed_record = "\n".join(splitted_record) + '\n'
        return processed_record

    def is_empty_removed_commit(self) -> bool:
        return (not self.removed_commit_hash) or (self.removed_commit_hash == GIT_HISTORY_NOT_BEEN_REMOVED)

    def _get_secret_validation_status_message(self) -> str:
        message = None
        if self.validation_status:
            message = TEXT_BY_SECRET_VALIDATION_STATUS.get(self.validation_status)

            if not message and self.validation_status != ValidationStatus.UNAVAILABLE.value:
                logging.debug(f'Got empty message for secret validation status = {self.validation_status}')

        return message or ''



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class GlobalSchemeDefineHTTP(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_18"
        name = "Ensure that global schemes use 'https' protocol instead of 'http'- version 2.0 files"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ['schemes']
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_resources,
                         block_type=BlockType.DOCUMENT)

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        schemes = conf.get("schemes", [])
        if not schemes:
            # If the schemes is not included, the default scheme to be used is the one used to access the Swagger
            # definition itself, in which case the current check is not relevant.
            return CheckResult.UNKNOWN, conf
        if 'http' in schemes:
            return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf


check = GlobalSchemeDefineHTTP()



from __future__ import annotations

import datetime
import hashlib
import linecache
import logging
import os
import re
from pathlib import Path
from typing import TYPE_CHECKING, cast, Optional, Iterable, Any, List

import requests
from detect_secrets.filters.heuristic import is_potential_uuid

from checkov.common.util.decorators import time_it
from checkov.common.util.type_forcers import convert_str_to_bool

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.output.secrets_record import SecretsRecord
from checkov.common.util.http_utils import request_wrapper
from detect_secrets import SecretsCollection
from detect_secrets.core import scan
from detect_secrets.settings import transient_settings

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    integration as metadata_integration
from checkov.common.bridgecrew.severities import Severity
from checkov.common.comment.enum import COMMENT_REGEX
from checkov.common.models.consts import SUPPORTED_FILE_EXTENSIONS
from checkov.common.models.enums import CheckResult
from checkov.common.output.report import Report
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
from checkov.common.runners.base_runner import ignored_directories
from checkov.common.typing import _CheckResult
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.dockerfile import is_docker_file
from checkov.common.util.secrets import omit_secret_value_from_line
from checkov.runner_filter import RunnerFilter
from checkov.secrets.consts import ValidationStatus, VerifySecretsResult
from checkov.secrets.coordinator import EnrichedSecret, SecretsCoordinator
from checkov.secrets.plugins.load_detectors import get_runnable_plugins
from checkov.secrets.scan_git_history import GitHistoryScanner

if TYPE_CHECKING:
    from checkov.common.util.tqdm_utils import ProgressBar
    from detect_secrets.core.potential_secret import PotentialSecret

SOURCE_CODE_EXTENSION = ['.py', '.js', '.properties', '.pem', '.php', '.xml', '.ts', '.env', '.java', '.rb',
                         'go', 'cs', '.txt']
SECRET_TYPE_TO_ID = {
    'Artifactory Credentials': 'CKV_SECRET_1',
    'AWS Access Key': 'CKV_SECRET_2',
    'Azure Storage Account access key': 'CKV_SECRET_3',
    'Basic Auth Credentials': 'CKV_SECRET_4',
    'Cloudant Credentials': 'CKV_SECRET_5',
    'Base64 High Entropy String': 'CKV_SECRET_6',
    'IBM Cloud IAM Key': 'CKV_SECRET_7',
    'IBM COS HMAC Credentials': 'CKV_SECRET_8',
    'JSON Web Token': 'CKV_SECRET_9',
    'Secret Keyword': 'CKV_SECRET_10',
    'Mailchimp Access Key': 'CKV_SECRET_11',
    'NPM tokens': 'CKV_SECRET_12',
    'Private Key': 'CKV_SECRET_13',
    'Slack Token': 'CKV_SECRET_14',
    'SoftLayer Credentials': 'CKV_SECRET_15',
    'Square OAuth Secret': 'CKV_SECRET_16',
    'Stripe Access Key': 'CKV_SECRET_17',
    'Twilio API Key': 'CKV_SECRET_18',
    'Hex High Entropy String': 'CKV_SECRET_19'
}
CHECK_ID_TO_SECRET_TYPE = {v: k for k, v in SECRET_TYPE_TO_ID.items()}

PROHIBITED_FILES = ['Pipfile.lock', 'yarn.lock', 'package-lock.json', 'requirements.txt']

MAX_FILE_SIZE = int(os.getenv('CHECKOV_MAX_FILE_SIZE', '5000000'))  # 5 MB is default limit


class Runner(BaseRunner[None]):
    check_type = CheckType.SECRETS  # noqa: CCE003  # a static attribute

    def __init__(self, file_extensions: Iterable[str] | None = None, file_names: Iterable[str] | None = None):
        super().__init__(file_extensions, file_names)
        self.secrets_coordinator = SecretsCoordinator()

    def run(
            self,
            root_folder: str | None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True
    ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        current_dir = Path(__file__).parent
        secrets = SecretsCollection()
        plugins_used = [
            {'name': 'AWSKeyDetector'},
            {'name': 'ArtifactoryDetector'},
            {'name': 'AzureStorageKeyDetector'},
            {'name': 'BasicAuthDetector'},
            {'name': 'CloudantDetector'},
            {'name': 'IbmCloudIamDetector'},
            {'name': 'MailchimpDetector'},
            {'name': 'PrivateKeyDetector'},
            {'name': 'SlackDetector'},
            {'name': 'SoftlayerDetector'},
            {'name': 'SquareOAuthDetector'},
            {'name': 'StripeDetector'},
            {'name': 'TwilioKeyDetector'},
            {'name': 'EntropyKeywordCombinator', 'path': f'file://{current_dir}/plugins/entropy_keyword_combinator.py'}
        ]

        # load runnable plugins
        customer_run_config = bc_integration.customer_run_config_response
        plugins_index = 0
        work_path = str(os.getenv('WORKDIR', current_dir))
        if customer_run_config:
            policies_list = customer_run_config.get('secretsPolicies', [])
            if policies_list:
                runnable_plugins: dict[str, str] = get_runnable_plugins(policies_list)
                logging.info(f"Found {len(runnable_plugins)} runnable plugins")
                if len(runnable_plugins) > 0:
                    plugins_index += 1
                for name, runnable_plugin in runnable_plugins.items():
                    f = open(f"{work_path}/runnable_plugin_{plugins_index}.py", "w")
                    f.write(runnable_plugin)
                    f.close()
                    plugins_used.append({
                        'name': name.replace(' ', ''),
                        'path': f'file://{work_path}/runnable_plugin_{plugins_index}.py'
                    })
                    plugins_index += 1
                    logging.info(f"Loaded runnable plugin {name}")
        # load internal regex detectors
        detector_path = f"{current_dir}/plugins/custom_regex_detector.py"
        logging.info(f"Custom detector found at {detector_path}. Loading...")
        plugins_used.append({
            'name': 'CustomRegexDetector',
            'path': f'file://{detector_path}'
        })
        with transient_settings({
            # Only run scans with only these plugins.
            'plugins_used': plugins_used
        }) as settings:
            report = Report(self.check_type)
            if not runner_filter.show_progress_bar:
                self.pbar.turn_off_progress_bar()

            # Implement non IaC files (including .terraform dir)
            files_to_scan = files or []
            excluded_paths = (runner_filter.excluded_paths or []) + ignored_directories + [DEFAULT_EXTERNAL_MODULES_DIR]
            self._add_custom_detectors_to_metadata_integration()
            if root_folder:
                if runner_filter.enable_git_history_secret_scan:
                    git_history_scanner = GitHistoryScanner(root_folder, secrets, runner_filter.git_history_timeout)
                    settings.disable_filters(*['detect_secrets.filters.common.is_invalid_file'])
                    git_history_scanner.scan_history(last_commit_scanned=runner_filter.git_history_last_commit_scanned)
                    logging.info(f'Secrets scanning git history for root folder {root_folder}')
                else:
                    enable_secret_scan_all_files = runner_filter.enable_secret_scan_all_files
                    block_list_secret_scan = runner_filter.block_list_secret_scan or []
                    block_list_secret_scan_lower = [file_type.lower() for file_type in block_list_secret_scan]
                    for root, d_names, f_names in os.walk(root_folder):
                        filter_ignored_paths(root, d_names, excluded_paths)
                        filter_ignored_paths(root, f_names, excluded_paths)
                        for file in f_names:
                            if enable_secret_scan_all_files:
                                if is_docker_file(file):
                                    if 'dockerfile' not in block_list_secret_scan_lower:
                                        files_to_scan.append(os.path.join(root, file))
                                elif f".{file.split('.')[-1]}" not in block_list_secret_scan_lower:
                                    files_to_scan.append(os.path.join(root, file))
                            elif file not in PROHIBITED_FILES and f".{file.split('.')[-1]}" in SUPPORTED_FILE_EXTENSIONS or is_docker_file(
                                    file):
                                files_to_scan.append(os.path.join(root, file))
                    logging.info(f'Secrets scanning will scan {len(files_to_scan)} files')

            settings.disable_filters(*['detect_secrets.filters.heuristic.is_indirect_reference'])
            settings.disable_filters(*['detect_secrets.filters.heuristic.is_potential_uuid'])

            if not runner_filter.enable_git_history_secret_scan:
                self.pbar.initiate(len(files_to_scan))
                self._scan_files(files_to_scan, secrets, self.pbar)
                self.pbar.close()
            secrets_duplication: dict[str, bool] = {}

            for key, secret in secrets:
                if runner_filter.enable_git_history_secret_scan:
                    added_commit_hash, removed_commit_hash, code_line = \
                        git_history_scanner.secret_store.get_added_and_removed_commit_hash(key, secret)
                else:
                    added_commit_hash, removed_commit_hash, code_line = None, None, None
                check_id = getattr(secret, "check_id", SECRET_TYPE_TO_ID.get(secret.type))
                if not check_id:
                    logging.debug(f'Secret was filtered - no check_id for line_number {secret.line_number}')
                    continue
                secret_key = f'{key}_{secret.line_number}_{secret.secret_hash}'
                if secret.secret_value and is_potential_uuid(secret.secret_value):
                    logging.info(
                        f"Removing secret due to UUID filtering: {hashlib.sha256(secret.secret_value.encode('utf-8')).hexdigest()}")
                    continue
                if secret_key in secrets_duplication:
                    logging.debug(
                        f'Secret was filtered - secrets_duplication. line_number {secret.line_number}, check_id {check_id}')
                    continue
                else:
                    secrets_duplication[secret_key] = True
                bc_check_id = metadata_integration.get_bc_id(check_id)
                severity = metadata_integration.get_severity(check_id)
                if not runner_filter.should_run_check(check_id=check_id, bc_check_id=bc_check_id, severity=severity,
                                                      report_type=CheckType.SECRETS):
                    logging.debug(
                        f'Check was suppress - should_run_check. check_id {check_id}')
                    continue
                result: _CheckResult = {'result': CheckResult.FAILED}
                try:
                    if runner_filter.enable_git_history_secret_scan and code_line is not None:
                        line_text = code_line
                    else:
                        line_text = linecache.getline(secret.filename, secret.line_number)
                except SyntaxError as e:
                    # If encoding is a problem, this is probably not human-readable source code
                    # hence there's no need in flagging this secret
                    logging.info(f'Failed to log secret {secret.type} for file {secret.filename} because of {e}')
                    continue
                if line_text and line_text.startswith('git_commit'):
                    continue
                result = self.search_for_suppression(
                    check_id=check_id,
                    bc_check_id=bc_check_id,
                    severity=severity,
                    secret=secret,
                    runner_filter=runner_filter,
                    root_folder=root_folder
                ) or result
                relative_file_path = f'/{os.path.relpath(secret.filename, root_folder)}'
                resource = f'{relative_file_path}:{added_commit_hash}:{secret.secret_hash}' if added_commit_hash else f'{relative_file_path}:{secret.secret_hash}'
                report.add_resource(resource)
                # 'secret.secret_value' can actually be 'None', but only when 'PotentialSecret' was created
                # via 'load_secret_from_dict'
                self.save_secret_to_coordinator(secret.secret_value, bc_check_id, resource, secret.line_number, result)
                line_text_censored = omit_secret_value_from_line(cast(str, secret.secret_value), line_text)
                report.add_record(SecretsRecord(
                    check_id=check_id,
                    bc_check_id=bc_check_id,
                    severity=severity,
                    check_name=secret.type,
                    check_result=result,
                    code_block=[(secret.line_number, line_text_censored)],
                    file_path=relative_file_path,
                    file_line_range=[secret.line_number, secret.line_number + 1],
                    resource=secret.secret_hash,
                    check_class="",
                    evaluations=None,
                    file_abs_path=os.path.abspath(secret.filename),
                    validation_status=ValidationStatus.UNAVAILABLE.value,
                    added_commit_hash=added_commit_hash,
                    removed_commit_hash=removed_commit_hash
                ))

            enriched_secrets_s3_path = bc_integration.persist_enriched_secrets(self.secrets_coordinator.get_secrets())
            if enriched_secrets_s3_path:
                self.verify_secrets(report, enriched_secrets_s3_path)
            logging.debug(f'report fail checks len: {len(report.failed_checks)}')

            self.cleanup_plugin_files(work_path, plugins_index)
            if runner_filter.skip_invalid_secrets:
                self._modify_invalid_secrets_check_result_to_skipped(report)
            return report

    def cleanup_plugin_files(self, work_path: str, amount: int) -> None:
        for index in range(1, amount):
            try:
                os.remove(f"{work_path}/runnable_plugin_{index}.py")
                logging.info(f"Removed runnable plugin at index {index}")
            except Exception as e:
                logging.info(f"Failed removing file at index {index} due to: {e}")

    @staticmethod
    def _scan_files(files_to_scan: list[str], secrets: SecretsCollection, pbar: ProgressBar) -> None:
        # implemented the scan function like secrets.scan_files
        base_path = secrets.root
        results = parallel_runner.run_function(
            func=lambda f: Runner._safe_scan(f, base_path),
            items=files_to_scan,
            run_multiprocess=os.getenv("RUN_SECRETS_MULTIPROCESS", "").lower() == "true"
        )

        for filename, secrets_results in results:
            pbar.set_additional_data({'Current File Scanned': str(filename)})
            for secret in secrets_results:
                secrets[os.path.relpath(secret.filename, base_path)].add(secret)
            pbar.update()

    @staticmethod
    def _safe_scan(file_path: str, base_path: str) -> tuple[str, list[PotentialSecret]]:
        full_file_path = os.path.join(base_path, file_path)
        file_size = os.path.getsize(full_file_path)
        if file_size > MAX_FILE_SIZE > 0:
            logging.info(
                f'Skipping secret scanning on {full_file_path} due to file size. To scan this file for '
                'secrets, run this command again with the environment variable "CHECKOV_MAX_FILE_SIZE" '
                f'to 0 or {file_size + 1}'
            )
            return file_path, []
        try:
            start_time = datetime.datetime.now()
            file_results = [*scan.scan_file(full_file_path)]
            logging.info(f'file {full_file_path} results len {len(file_results)}')
            end_time = datetime.datetime.now()
            run_time = end_time - start_time
            if run_time > datetime.timedelta(seconds=10):
                logging.info(f'Secret scanning for {full_file_path} took {run_time} seconds')
            return file_path, file_results
        except Exception as e:
            logging.warning(f"Secret scanning: could not process file {full_file_path}")
            logging.debug(e, exc_info=True)
            return file_path, []

    @staticmethod
    def search_for_suppression(
            check_id: str,
            bc_check_id: str,
            severity: Severity | None,
            secret: PotentialSecret,
            runner_filter: RunnerFilter,
            root_folder: str | None
    ) -> _CheckResult | None:
        if not runner_filter.should_run_check(
                check_id=check_id,
                bc_check_id=bc_check_id,
                severity=severity,
                report_type=CheckType.SECRETS,
                file_origin_paths=[secret.filename],
                root_folder=root_folder
        ) and check_id in CHECK_ID_TO_SECRET_TYPE.keys():
            return {
                "result": CheckResult.SKIPPED,
                "suppress_comment": f"Secret scan {check_id} is skipped"
            }

        # Check for suppression comment in the line before, the line of, and the line after the secret
        for line_number in [secret.line_number, secret.line_number - 1, secret.line_number + 1]:
            lt = linecache.getline(secret.filename, line_number)
            skip_search = re.search(COMMENT_REGEX, lt)
            if skip_search and (skip_search.group(2) == check_id or skip_search.group(2) == bc_check_id):
                return {
                    "result": CheckResult.SKIPPED,
                    "suppress_comment": skip_search.group(3)[1:] if skip_search.group(3) else "No comment provided"
                }
        return None

    def save_secret_to_coordinator(
            self, secret_value: Optional[str], bc_check_id: str, resource: str, line_number: int, result: _CheckResult
    ) -> None:
        if result.get('result') == CheckResult.FAILED and secret_value is not None:
            enriched_secret = EnrichedSecret(
                original_secret=secret_value, bc_check_id=bc_check_id, resource=resource, line_number=line_number
            )
            self.secrets_coordinator.add_secret(enriched_secret=enriched_secret)

    @time_it
    def verify_secrets(self, report: Report, enriched_secrets_s3_path: str) -> VerifySecretsResult:
        if not bc_integration.bc_api_key or not convert_str_to_bool(os.getenv("CKV_VALIDATE_SECRETS", False)):
            logging.debug(
                'Secrets verification is off, enabled it via env var CKV_VALIDATE_SECRETS and provide an api key')
            return VerifySecretsResult.INSUFFICIENT_PARAMS

        if bc_integration.skip_download:
            logging.debug('Skipping secrets verification as flag skip-download was specified')
            return VerifySecretsResult.INSUFFICIENT_PARAMS

        request_body = {
            "reportS3Path": enriched_secrets_s3_path
        }
        response = None
        try:
            response = request_wrapper(
                "POST", f"{bc_integration.api_url}/api/v1/secrets/reportVerification",
                headers=bc_integration.get_default_headers("POST"),
                json=request_body,
                should_call_raise_for_status=True,
                log_json_body=False
            )
        except Exception:
            logging.error('Failed to perform secrets verification', exc_info=True)

        if not response:
            return VerifySecretsResult.FAILURE

        verification_report_presigned_url = response.json().get('verificationReportSignedUrl')
        if not verification_report_presigned_url:
            logging.error("Response is missing verificationReportSignedUrl key, aborting")
            return VerifySecretsResult.FAILURE

        verification_report = self.get_json_verification_report(verification_report_presigned_url)

        if not verification_report:
            return VerifySecretsResult.FAILURE

        validation_status_by_check_id_and_resource = {}
        for validation_status_entity in verification_report:
            if not all(required_key in validation_status_entity.keys() for required_key in
                       ["violationId", "resourceId", "status"]):
                logging.debug(f"{validation_status_entity} does not have all required keys, skipping")
                continue

            key = f'{validation_status_entity["violationId"]}_{validation_status_entity["resourceId"]}'
            validation_status_by_check_id_and_resource[key] = validation_status_entity['status']

        logging.debug(
            f'secrets verification api returned with {len(validation_status_by_check_id_and_resource.keys())} unique entries')

        for secrets_record in report.failed_checks:
            if hasattr(secrets_record, "validation_status"):
                key = f'{secrets_record.bc_check_id}_{secrets_record.file_path}:{secrets_record.resource}'
                secrets_record.validation_status = validation_status_by_check_id_and_resource.get(key)

                if secrets_record.validation_status is None:
                    logging.debug(f'Failed to find verification status of {key}, setting by default to Unknown')
                    secrets_record.validation_status = ValidationStatus.UNAVAILABLE.value

        return VerifySecretsResult.SUCCESS

    @staticmethod
    def get_json_verification_report(presigned_url: str) -> list[dict[str, str]] | None:
        response = None
        try:
            response = requests.get(presigned_url)
        except Exception:
            logging.error('Unable to download verification report')

        return response.json() if response else None

    @staticmethod
    def _add_custom_detectors_to_metadata_integration() -> None:
        customer_run_config_response = bc_integration.customer_run_config_response
        policies_list: List[dict[str, Any]] = []
        if customer_run_config_response:
            policies_list = customer_run_config_response.get('secretsPolicies', [])
        for policy in policies_list:
            if policy.get('isCustom', False):
                check_id = policy['incidentId']
                guideline = policy.get('guideline', '')
                severity = policy.get('severity', '')
                metadata_integration.check_metadata[check_id] = {'id': check_id,
                                                                 'guideline': guideline,
                                                                 'severity': severity}

    @staticmethod
    def _modify_invalid_secrets_check_result_to_skipped(report: Report) -> None:
        checks_indexes_moved_to_skipped: list[int] = []

        for check_index, check in enumerate(report.failed_checks):
            if hasattr(check, 'validation_status') and check.validation_status == ValidationStatus.INVALID.value:
                check.check_result["result"] = CheckResult.SKIPPED
                check.check_result["suppress_comment"] = "Skipped invalid secret"
                report.skipped_checks.append(check)
                checks_indexes_moved_to_skipped.append(check_index)

        for idx in sorted(checks_indexes_moved_to_skipped, reverse=True):
            try:
                del report.failed_checks[idx]
            except Exception:
                logging.error(f"Failed to remove suppressed secrets violations from failed_checks, report is corrupted."
                              f"Tried to delete entry {idx} from failed_checks of length {len(report.failed_checks)}",
                              exc_info=True)



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class AppServiceIdentity(BaseResourceCheck):
    def __init__(self) -> None:
        # https://docs.microsoft.com/en-us/azure/templates/microsoft.web/2019-08-01/sites#ManagedServiceIdentity
        # https://docs.microsoft.com/en-us/azure/app-service/overview-managed-identity
        # https://docs.microsoft.com/en-us/azure/app-service/samples-resource-manager-templates
        name = "Ensure that Register with Azure Active Directory is enabled on App Service"
        id = "CKV_AZURE_16"
        supported_resources = ('Microsoft.Web/sites',)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "identity" in conf:
            if "type" in conf["identity"]:
                if conf["identity"]["type"] == "SystemAssigned":
                    return CheckResult.PASSED
                elif conf["identity"]["type"] == "UserAssigned":
                    if "userAssignedIdentities" in conf["identity"]:
                        if conf["identity"]["userAssignedIdentities"]:
                            return CheckResult.PASSED
        return CheckResult.FAILED


check = AppServiceIdentity()



from checkov.common.models.consts import ANY_VALUE
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class AppServiceHttpLoggingEnabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that App service enables HTTP logging"
        id = "CKV_AZURE_63"
        supported_resources = ('azurerm_app_service', 'azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "logs/[0]/http_logs"

    def get_expected_value(self):
        return ANY_VALUE


check = AppServiceHttpLoggingEnabled()



from __future__ import annotations

import logging
import json
import re
from copy import deepcopy
from typing import Any, Optional, TypeVar

from checkov.common.graph.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.graph_builder.variable_rendering.renderer import TerraformVariableRenderer
import checkov.terraform.graph_builder.local_graph as l_graph
from checkov.terraform.graph_builder.variable_rendering.evaluate_terraform import evaluate_terraform

FOREACH_STRING = 'for_each'
COUNT_STRING = 'count'
REFERENCES_VALUES = r"(var|module|local)\."
FOR_EACH_BLOCK_TYPE = TypeVar("FOR_EACH_BLOCK_TYPE", bound="dict[int, Optional[list[str] | dict[str, Any] | int]]")
COUNT_KEY = 'count.index'
EACH_KEY = 'each.key'
EACH_VALUE = 'each.value'


class ForeachHandler(object):
    def __init__(self, local_graph: l_graph.TerraformLocalGraph) -> None:
        self.local_graph = local_graph

    def handle_foreach_rendering(self, foreach_blocks: dict[str, list[int]]) -> None:
        # handle_foreach_rendering_for_module(foreach_blocks.get(BlockType.MODULE))
        self._handle_foreach_rendering_for_resource(foreach_blocks.get(BlockType.RESOURCE))

    def _handle_foreach_rendering_for_resource(self, resources_blocks: list[int]) -> None:
        block_index_to_statement = self._get_statements(resources_blocks)
        self._create_new_resources(block_index_to_statement)

    def _get_statements(self, resources_blocks: list[int]) -> FOR_EACH_BLOCK_TYPE:
        block_index_to_statement: FOR_EACH_BLOCK_TYPE = {}
        for block_index in resources_blocks:
            foreach_statement = self._get_static_foreach_statement(block_index)
            block_index_to_statement[block_index] = foreach_statement
        blocks_to_render = [block_idx for block_idx, statement in block_index_to_statement.items() if statement is None]
        if blocks_to_render:
            rendered_statements = self._handle_dynamic_statement(blocks_to_render)
            block_index_to_statement.update(rendered_statements)
        return block_index_to_statement

    def _get_static_foreach_statement(self, block_index: int) -> Optional[list[str] | dict[str, Any]]:
        attributes = self.local_graph.vertices[block_index].attributes
        if not attributes.get(FOREACH_STRING) and not attributes.get(COUNT_STRING):
            return
        try:
            if self._is_static_statement(block_index):
                return self._handle_static_statement(block_index)
            else:
                return None
        except Exception as e:
            logging.info(f"Cant get foreach statement for block: {self.local_graph.vertices[block_index]}, error: {str(e)}")
            return None

    def _is_static_foreach_statement(self, statement: list[str] | dict[str, Any]) -> bool:
        if isinstance(statement, list):
            statement = self.extract_from_list(statement)
        if isinstance(statement, str) and re.search(REFERENCES_VALUES, statement):
            return False
        if isinstance(statement, (list, dict)) and any([re.search(REFERENCES_VALUES, s) for s in statement]):
            return False
        return True

    def _is_static_count_statement(self, statement: list[str] | int) -> bool:
        if isinstance(statement, list):
            statement = self.extract_from_list(statement)
        if isinstance(statement, int):
            return True
        if isinstance(statement, str) and not re.search(REFERENCES_VALUES, statement):
            return True
        return False

    def _is_static_statement(self, block_index: int, sub_graph: Optional[l_graph.TerraformLocalGraph] = None) -> bool:
        """
        foreach statement can be list/map of strings or map, if its string we need to render it for sure.
        """
        block = self.local_graph.vertices[block_index] if not sub_graph else sub_graph.vertices[block_index]
        foreach_statement = evaluate_terraform(block.attributes.get(FOREACH_STRING))
        count_statement = evaluate_terraform(block.attributes.get(COUNT_STRING))
        if foreach_statement:
            return self._is_static_foreach_statement(foreach_statement)
        if count_statement:
            return self._is_static_count_statement(count_statement)
        return False

    @staticmethod
    def extract_from_list(val: list[str] | list[int]) -> list[str] | list[int] | int | str:
        return val[0] if len(val) == 1 and isinstance(val[0], (str, int)) else val

    def _handle_static_foreach_statement(self, statement: list[str] | dict[str, Any]) -> Optional[list[str] | dict[str, Any]]:
        if isinstance(statement, list):
            statement = self.extract_from_list(statement)
        evaluated_statement = evaluate_terraform(statement)
        if isinstance(evaluated_statement, str):
            try:
                evaluated_statement = json.loads(evaluated_statement)
            except ValueError:
                pass
        if isinstance(evaluated_statement, set):
            evaluated_statement = list(evaluated_statement)
        if isinstance(evaluated_statement, (dict, list)) and all(isinstance(val, str) for val in evaluated_statement):
            return evaluated_statement
        return

    def _handle_static_count_statement(self, statement: list[str] | int) -> Optional[int]:
        if isinstance(statement, list):
            statement = self.extract_from_list(statement)
        evaluated_statement = evaluate_terraform(statement)
        if isinstance(evaluated_statement, int):
            return evaluated_statement
        return

    def _handle_static_statement(self, block_index: int, sub_graph: Optional[l_graph.TerraformLocalGraph] = None) -> Optional[list[str] | dict[str, Any] | int]:
        attrs = self.local_graph.vertices[block_index].attributes if not sub_graph else sub_graph.vertices[block_index].attributes
        foreach_statement = attrs.get(FOREACH_STRING)
        count_statement = attrs.get(COUNT_STRING)
        if foreach_statement:
            return self._handle_static_foreach_statement(foreach_statement)
        if count_statement:
            return self._handle_static_count_statement(count_statement)
        return

    def _handle_dynamic_statement(self, blocks_to_render: list[int]) -> FOR_EACH_BLOCK_TYPE:
        rendered_statements_by_idx: FOR_EACH_BLOCK_TYPE = {}
        sub_graph = self._build_sub_graph(blocks_to_render)
        self._render_sub_graph(sub_graph, blocks_to_render)
        for block_idx in blocks_to_render:
            if not self._is_static_statement(block_idx, sub_graph):
                rendered_statements_by_idx[block_idx] = None
            else:
                rendered_statements_by_idx[block_idx] = self._handle_static_statement(block_idx, sub_graph)
        return rendered_statements_by_idx

    @staticmethod
    def _render_sub_graph(sub_graph: l_graph.TerraformLocalGraph, blocks_to_render: list[int]) -> None:
        renderer = TerraformVariableRenderer(sub_graph)
        renderer.vertices_index_to_render = blocks_to_render
        renderer.render_variables_from_local_graph()

    def _build_sub_graph(self, blocks_to_render: list[int]) -> l_graph.TerraformLocalGraph:
        module = deepcopy(self.local_graph.module)
        sub_graph = l_graph.TerraformLocalGraph(module)
        sub_graph.vertices = [{}] * len(self.local_graph.vertices)
        for i, block in enumerate(self.local_graph.vertices):
            if not (block.block_type == BlockType.RESOURCE and i not in blocks_to_render):
                sub_graph.vertices[i] = deepcopy(block)  # type: ignore
        sub_graph.edges = [
            deepcopy(edge) for edge in self.local_graph.edges if (sub_graph.vertices[edge.dest] and sub_graph.vertices[edge.origin])
        ]
        sub_graph.in_edges = deepcopy(self.local_graph.in_edges)
        sub_graph.out_edges = deepcopy(self.local_graph.out_edges)
        return sub_graph

    def _create_new_resources_count(self, statement: int, block_idx: int) -> None:
        main_resource = self.local_graph.vertices[block_idx]
        for i in range(statement):
            self._create_new_resource(main_resource, i, resource_idx=block_idx, foreach_idx=i)

    @staticmethod
    def _pop_foreach_attrs(attrs: dict[str, Any]) -> None:
        attrs.pop(COUNT_STRING, None)
        attrs.pop(FOREACH_STRING, None)

    @staticmethod
    def __update_str_attrs(attrs: dict[str, Any], key_to_change: str, val_to_change: str, k: str | int) -> bool:
        if attrs[k] == "${" + key_to_change + "}":
            attrs[k] = val_to_change
            return True
        else:
            attrs[k] = attrs[k].replace("${" + key_to_change + "}", str(val_to_change))
            attrs[k] = attrs[k].replace(key_to_change, str(val_to_change))
            return True

    def _update_attributes(self, attrs: dict[str, Any], key_to_val_changes: dict[str, Any]) -> list[str]:
        foreach_attributes: list[str] = []
        for key_to_change, val_to_change in key_to_val_changes.items():
            for k, v in attrs.items():
                v_changed = False
                if isinstance(v, str):
                    v_changed = self.__update_str_attrs(attrs, key_to_change, val_to_change, k)
                if isinstance(v, dict):
                    nested_attrs = self._update_attributes(v, {key_to_change: val_to_change})
                    foreach_attributes.extend([k + '.' + na for na in nested_attrs])
                if isinstance(v, list) and len(v) == 1 and isinstance(v[0], dict):
                    nested_attrs = self._update_attributes(v[0], {key_to_change: val_to_change})
                    foreach_attributes.extend([k + '.' + na for na in nested_attrs])
                elif isinstance(v, list) and len(v) == 1 and isinstance(v[0], str) and key_to_change in v[0]:
                    if attrs[k][0] == "${" + key_to_change + "}":
                        attrs[k][0] = val_to_change
                        v_changed = True
                    else:
                        attrs[k][0] = attrs[k][0].replace("${" + key_to_change + "}", str(val_to_change))
                        attrs[k][0] = attrs[k][0].replace(key_to_change, str(val_to_change))
                        v_changed = True
                elif isinstance(v, list) and len(v) == 1 and isinstance(v[0], list):
                    for i, item in enumerate(v):
                        if isinstance(item, str) and (key_to_change in item or "${" + key_to_change + "}" in item):
                            if v[i] == "${" + key_to_change + "}":
                                v[i] = val_to_change
                                v_changed = True
                            else:
                                v[i] = item.replace("${" + key_to_change + "}", str(val_to_change))
                                v[i] = v[i].replace(key_to_change, str(val_to_change))
                                v_changed = True
                if v_changed:
                    foreach_attributes.append(k)
        return foreach_attributes

    @staticmethod
    def _build_key_to_val_changes(new_val: str, new_key: str):
        return {
            EACH_VALUE: new_val,
            EACH_KEY: new_key
        }

    def _create_new_resource(
            self,
            main_resource: TerraformBlock,
            new_value: int | str,
            resource_idx: int,
            foreach_idx: int,
            new_key: Optional[str] = None,
    ):
        new_resource = deepcopy(main_resource)
        block_type, block_name = new_resource.name.split('.')
        if main_resource.attributes.get(COUNT_STRING):
            key_to_val_changes = {COUNT_KEY: new_value}
        else:
            key_to_val_changes = self._build_key_to_val_changes(new_value, new_key)
        config_attrs = new_resource.config.get(block_type, {}).get(block_name, {})
        self._pop_foreach_attrs(new_resource.attributes)
        self._pop_foreach_attrs(config_attrs)
        self._update_attributes(new_resource.attributes, key_to_val_changes)
        foreach_attrs = self._update_attributes(config_attrs, key_to_val_changes)
        new_resource.foreach_attrs = foreach_attrs

        idx_to_change = new_key or new_value
        self._add_index_to_block_properties(new_resource, idx_to_change)
        if foreach_idx == 0:
            self.local_graph.vertices[resource_idx] = new_resource
        else:
            self.local_graph.vertices.append(new_resource)

    def _create_new_resources_foreach(self, statement: list[str] | dict[str, Any], block_idx: int) -> None:
        main_resource = self.local_graph.vertices[block_idx]
        if isinstance(statement, list):
            for i, new_value in enumerate(statement):
                self._create_new_resource(main_resource, new_value, new_key=new_value, resource_idx=block_idx, foreach_idx=i)
        if isinstance(statement, dict):
            for i, (new_key, new_value) in enumerate(statement.items()):
                self._create_new_resource(main_resource, new_value, new_key=new_key, resource_idx=block_idx, foreach_idx=i)

    @staticmethod
    def _add_index_to_block_properties(block: TerraformBlock, idx: str | int) -> None:
        block_type, block_name = block.name.split('.')
        # Note it is important to use `\"` inside the string,
        # as the string `["` is the separator for `foreach` in terraform.
        # In `count` it is just `[`
        idx_with_separator = f'\"{idx}\"' if isinstance(idx, str) else f'{idx}'

        block.id = f"{block.id}[{idx_with_separator}]"
        block.name = f"{block.name}[{idx_with_separator}]"
        if block.config.get(block_type) and block.config.get(block_type, {}).get(block_name):
            block.config[block_type][f"{block_name}[{idx_with_separator}]"] = block.config[block_type].pop(block_name)

    def _create_new_resources(self, block_index_to_statement: FOR_EACH_BLOCK_TYPE) -> None:
        for block_idx, statement in block_index_to_statement.items():
            if not statement:
                continue
            if isinstance(statement, int):
                self._create_new_resources_count(statement, block_idx)
            else:
                self._create_new_resources_foreach(statement, block_idx)



from __future__ import annotations
import json
from typing import Optional, Any


class TFModule:
    __slots__ = ("path", "name", "foreach_idx", "nested_tf_module")

    def __init__(self, path: str, name: str, nested_tf_module: Optional[TFModule] = None,
                 foreach_idx: Optional[int | str] = None) -> None:
        self.path = path
        self.name = name
        self.foreach_idx = foreach_idx
        self.nested_tf_module = nested_tf_module

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, TFModule):
            return False
        return self.path == other.path and self.name == other.name and self.nested_tf_module == other.nested_tf_module and self.foreach_idx == other.foreach_idx

    def __lt__(self, other: Any) -> bool:
        if not isinstance(other, TFModule):
            return False
        return (self.path, self.name, self.nested_tf_module, self.foreach_idx) < (
            other.path, other.name, other.nested_tf_module, other.foreach_idx)

    def __repr__(self) -> str:
        return f'path:{self.path}, name:{self.name}, nested_tf_module:{self.nested_tf_module}, foreach_idx:{self.foreach_idx}'

    def __hash__(self) -> int:
        return hash((self.path, self.name, self.nested_tf_module, self.foreach_idx))

    def __iter__(self):
        yield from {
            "path": self.path,
            "name": self.name,
            "foreach_idx": self.foreach_idx,
            "nested_tf_module": dict(self.nested_tf_module) if self.nested_tf_module else None
        }.items()

    def __str__(self):
        from checkov.common.util.json_utils import CustomJSONEncoder
        return json.dumps(dict(self), cls=CustomJSONEncoder)

    @staticmethod
    def from_json(json_dct):
        return TFModule(path=json_dct['path'], name=json_dct['name'], foreach_idx=json_dct['foreach_idx'],
                        nested_tf_module=TFModule.from_json(json_dct['nested_tf_module']))


class TFDefinitionKey:
    __slots__ = ("tf_source_modules", "file_path")

    def __init__(self, file_path: str, tf_source_modules: Optional[TFModule] = None) -> None:
        self.tf_source_modules = tf_source_modules
        self.file_path = file_path

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, TFDefinitionKey):
            return False
        return self.tf_source_modules == other.tf_source_modules and self.file_path == other.file_path

    def __lt__(self, other: Any) -> bool:
        if not isinstance(other, TFDefinitionKey):
            return False
        return (self.file_path, self.tf_source_modules) < (other.file_path, other.tf_source_modules)

    def __repr__(self) -> str:
        return f'tf_source_modules:{self.tf_source_modules}, file_path:{self.file_path}'

    def __hash__(self) -> int:
        return hash((self.file_path, self.tf_source_modules))

    def __iter__(self):
        yield from {
            "file_path": self.file_path,
            "tf_source_modules": dict(self.tf_source_modules) if self.tf_source_modules else None
        }.items()

    def __str__(self):
        from checkov.common.util.json_utils import CustomJSONEncoder
        return json.dumps(self.to_json(), cls=CustomJSONEncoder)

    def to_json(self):
        to_return = {"file_path": self.file_path, "tf_source_modules": None}
        if self.tf_source_modules:
            to_return["tf_source_modules"] = dict(self.tf_source_modules)
        return to_return

    @staticmethod
    def from_json(json_dct):
        return TFDefinitionKey(file_path=json_dct['file_path'],
                               tf_source_modules=TFModule.from_json(json_dct['tf_source_modules']))



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class PrivilegedContainers(BaseResourceCheck):

    def __init__(self):
        # CIS-1.3 1.7.1
        # CIS-1.5 5.2.1
        name = "Do not admit privileged containers"
        id = "CKV_K8S_16"

        supported_resources = ['kubernetes_pod', "kubernetes_pod_v1",
                               'kubernetes_deployment', 'kubernetes_deployment_v1']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        spec = conf.get('spec', [None])[0]
        if not spec:
            return CheckResult.UNKNOWN

        evaluated_keys_path = "spec"

        template = spec.get("template")
        if template and isinstance(template, list):
            template = template[0]
            template_spec = template.get("spec")
            if template_spec and isinstance(template_spec, list):
                spec = template_spec[0]
                evaluated_keys_path = f'{evaluated_keys_path}/[0]/template/[0]/spec'

        if spec.get("container"):
            containers = spec.get("container")

            for idx, container in enumerate(containers):
                if type(container) != dict:
                    return CheckResult.UNKNOWN
                if container.get("security_context"):
                    context = container.get("security_context")[0]
                    if context.get("privileged") == [True]:
                        self.evaluated_keys = [f'{evaluated_keys_path}/[0]/container/[{idx}]/security_context/[0]/privileged']
                        return CheckResult.FAILED
        return CheckResult.PASSED


check = PrivilegedContainers()



from __future__ import annotations

import logging
import re
from collections.abc import Collection
from pathlib import Path
from typing import Any

from checkov.terraform_json.parser import parse

TF_JSON_POSSIBLE_FILE_ENDINGS = (".tf.json",)

logger = logging.getLogger(__name__)


def get_scannable_file_paths(
    root_folder: str | Path | None = None, files: list[str] | None = None, excluded_paths: list[str] | None = None
) -> set[Path]:
    """Finds Terraform JSON files"""

    file_paths: set[Path] = set()

    if root_folder:
        root_path = Path(root_folder)
        file_paths = {
            file_path
            for file_ending in TF_JSON_POSSIBLE_FILE_ENDINGS
            for file_path in root_path.rglob(f"*{file_ending}")
            if file_path.is_file()
        }

        if excluded_paths:
            compiled = [re.compile(p.replace(".terraform", r"\.terraform")) for p in excluded_paths]
            file_paths = {
                file_path for file_path in file_paths if not any(pattern.search(str(file_path)) for pattern in compiled)
            }
    if files:
        for file in files:
            if file.endswith(TF_JSON_POSSIBLE_FILE_ENDINGS):
                file_paths.add(Path(file))

    return file_paths


def create_definitions(
    file_paths: Collection[Path],
) -> tuple[dict[str, dict[str, Any]], dict[str, list[tuple[int, str]]], list[str]]:
    """Creates dict objects and code lines for given files"""

    logger.info(f"Start to parse {len(file_paths)} files")

    definitions: "dict[str, dict[str, Any]]" = {}
    definitions_raw: "dict[str, list[tuple[int, str]]]" = {}
    parsing_errors: "list[str]" = []

    for file_path in file_paths:
        template, file_lines = parse(file_path)
        if template and file_lines:
            file_path_str = str(file_path)
            definitions[file_path_str] = template
            definitions_raw[file_path_str] = file_lines
        else:
            parsing_errors.append(str(file_path.resolve()))

    logging.info(f"Successfully parsed {len(definitions)} files")

    return definitions, definitions_raw, parsing_errors



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceSetHealthCheck(BaseResourceValueCheck):
    def __init__(self) -> None:
        # "Azure App Service monitors a specific path for each web app instance to determine health status.
        # The monitored path should implement functional checks to determine if the app is performing correctly.
        # The checks should include dependencies including those that may not be regularly called.
        # Regular checks of the monitored path allow Azure App Service to route traffic based on availability."
        name = "Ensure that App Service configures health check"
        id = "CKV_AZURE_213"
        supported_resources = ('azurerm_app_service', 'azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'site_config/[0]/health_check_path'

    def get_expected_values(self) -> Any:
        return ANY_VALUE


check = AppServiceSetHealthCheck()



from checkov.terraform.checks.resource.aws import *  # noqa
from checkov.terraform.checks.resource.gcp import *  # noqa
from checkov.terraform.checks.resource.azure import *  # noqa
from checkov.terraform.checks.resource.github import *  # noqa
from checkov.terraform.checks.resource.gitlab import *  # noqa
from checkov.terraform.checks.resource.linode import *  # noqa
from checkov.terraform.checks.resource.oci import *  # noqa
from checkov.terraform.checks.resource.openstack import *  # noqa
from checkov.terraform.checks.resource.panos import *  # noqa
from checkov.terraform.checks.resource.digitalocean import *  # noqa
from checkov.terraform.checks.resource.alicloud import *  # noqa
from checkov.terraform.checks.resource.kubernetes import *  # noqa
from checkov.terraform.checks.resource.yandexcloud import *  # noqa
from checkov.terraform.checks.resource.ncp import *  # noqa



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class ApiServerNamespaceLifecyclePlugin(BaseK8sContainerCheck):
    def __init__(self) -> None:
        id = "CKV_K8S_83"
        name = "Ensure that the admission control plugin NamespaceLifecycle is set"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        if conf.get("command"):
            if "kube-apiserver" in conf["command"]:
                for cmd in conf["command"]:
                    if cmd == "--enable-admission-plugins":
                        return CheckResult.FAILED
                    if "=" in cmd:
                        [field, value, *_] = cmd.split("=")
                        if field == "--enable-admission-plugins":
                            if "NamespaceLifecycle" not in value:
                                return CheckResult.FAILED
        return CheckResult.PASSED


check = ApiServerNamespaceLifecyclePlugin()



from __future__ import annotations

from typing import Any


def extract_commands(conf: dict[str, Any]) -> tuple[list[str], list[str]]:
    commands = conf.get("command")
    if not commands or not isinstance(commands, list):
        return [], []
    values = []
    keys = []
    for cmd in commands:
        if cmd is None:
            continue
        if "=" in cmd:
            key, value = cmd.split("=", maxsplit=1)
            keys.append(key)
            values.append(value)
        else:
            keys.append(cmd)
            values.append(None)
    return keys, values



from typing_extensions import TypedDict, Protocol


class ExampleTypedDict(TypedDict):
    attr_1: str
    attr_2: int


class ExampleProtocol(Protocol):
    attr: str

    def do_something(self) -> str:
        ...



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class CloudSqlMajorVersion(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure SQL database is using latest Major version"
        id = "CKV_GCP_79"
        supported_resources = ['google_sql_database_instance']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'database_version'

    def get_expected_values(self):
        return ["POSTGRES_14", "MYSQL_8_0", "SQLSERVER_2019_STANDARD", "SQLSERVER_2019_WEB",
                "SQLSERVER_2019_ENTERPRISE", "SQLSERVER_2019_EXPRESS"]


check = CloudSqlMajorVersion()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class DockerSocketVolume(BaseK8Check):
    def __init__(self) -> None:
        name = "Do not expose the docker daemon socket to containers"
        # Exposing the socket gives container information and increases risk of exploit
        # read-only is not a solution but only makes it harder to exploit.
        # Location: Pod.spec.volumes[].hostPath.path
        # Location: CronJob.spec.jobTemplate.spec.template.spec.volumes[].hostPath.path
        # Location: *.spec.template.spec.volumes[].hostPath.path
        id = "CKV_K8S_27"
        supported_kind = (
            "Pod",
            "Deployment",
            "DaemonSet",
            "StatefulSet",
            "ReplicaSet",
            "ReplicationController",
            "Job",
            "CronJob",
        )
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        spec = {}

        if conf["kind"] == "Pod":
            if "spec" in conf:
                spec = conf["spec"]
        elif conf["kind"] == "CronJob":
            spec = conf.get("spec")
            if spec:
                job_template = spec.get("jobTemplate")
                if job_template:
                    job_template_spec = job_template.get("spec")
                    if job_template_spec:
                        template = job_template_spec.get("template")
                        if template:
                            if "spec" in template:
                                spec = template["spec"]
        else:
            inner_spec = self.get_inner_entry(conf, "spec")
            spec = inner_spec if inner_spec else spec

        # Evaluate volumes
        if spec and isinstance(spec, dict):
            volumes = spec.get("volumes", [])
            if not isinstance(volumes, list):
                return CheckResult.UNKNOWN
            for v in volumes:
                if not v.get("hostPath"):
                    continue
                if v["hostPath"].get("path") == "/var/run/docker.sock":
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = DockerSocketVolume()



from __future__ import annotations

import re
from typing import Iterable
from typing import Any

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories
from checkov.github.dal import CKV_METADATA
from checkov.github.registry import registry


HTTP = re.compile("^http://")


class BaseGithubCheck(BaseCheck):
    def __init__(self, name: str, id: str, categories: Iterable[CheckCategories], supported_entities: Iterable[str],
                 block_type: str, path: str | None = None, guideline: str | None = None) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type=block_type,
            guideline=guideline,
        )
        self.path = path
        registry.register(self)

    @staticmethod
    def resolve_ckv_metadata_conf(conf: dict[str, Any]) -> tuple[dict[str, Any], dict[str, Any]]:
        if isinstance(conf, list) and conf:
            ckv_metadata = conf[-1]
            new_conf = conf[:-1]
            return ckv_metadata, new_conf
        elif isinstance(conf, dict):
            ckv_metadata = conf.get(CKV_METADATA)
            if ckv_metadata:
                new_conf = conf.copy()
                del new_conf[CKV_METADATA]
                return ckv_metadata, new_conf
        return {}, conf



import logging

from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.common.output.report import Report
from checkov.terraform.plan_parser import TF_PLAN_RESOURCE_ADDRESS
from typing import Dict


class DeepAnalysisGraphManager:
    def __init__(self, tf_graph: TerraformLocalGraph, tf_plan_graph: TerraformLocalGraph) -> None:
        self.tf_graph: TerraformLocalGraph = tf_graph
        self.tf_plan_graph: TerraformLocalGraph = tf_plan_graph
        self._address_to_tf_vertex_map: Dict[str, TerraformBlock] = {}
        self._address_to_tf_plan_vertex_map: Dict[str, TerraformBlock] = {}
        self._apply_address_mapping()

    def _apply_address_mapping(self) -> None:
        self._address_to_tf_vertex_map = {
            vertex.attributes[TF_PLAN_RESOURCE_ADDRESS]: vertex
            for vertex in self.tf_graph.vertices
            if vertex.block_type == BlockType.RESOURCE
        }
        self._address_to_tf_plan_vertex_map = {
            vertex.attributes[TF_PLAN_RESOURCE_ADDRESS]: vertex
            for vertex in self.tf_plan_graph.vertices
            if vertex.block_type == BlockType.RESOURCE
        }

    def enrich_tf_graph_attributes(self) -> None:
        for address, tf_plan_vertex in self._address_to_tf_plan_vertex_map.items():
            tf_vertex = self._address_to_tf_vertex_map.get(address)
            if not tf_vertex:
                logging.info(f'Cant find this address: {address} in tf graph')
                continue
            tf_vertex.attributes = {**tf_vertex.attributes, **tf_plan_vertex.attributes}
            tf_vertex.path = tf_plan_vertex.path

    def filter_report(self, report: Report) -> None:
        report.failed_checks = [check for check in report.failed_checks if
                                check.resource_address in self._address_to_tf_plan_vertex_map]
        report.passed_checks = [check for check in report.passed_checks if
                                check.resource_address in self._address_to_tf_plan_vertex_map]
        report.skipped_checks = [check for check in report.skipped_checks if
                                 check.resource_address in self._address_to_tf_plan_vertex_map]
        # No need to filter other fields for now
        report.resources = set()
        report.extra_resources = set()
        report.parsing_errors = []



from checkov.arm.checks.resource.NSGRulePortAccessRestricted import NSGRulePortAccessRestricted


class NSGRuleRDPAccessRestricted(NSGRulePortAccessRestricted):
    def __init__(self) -> None:
        super().__init__(
            name="Ensure that RDP access is restricted from the internet", check_id="CKV_AZURE_9", port=3389
        )


check = NSGRuleRDPAccessRestricted()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SQSQueueEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure all data stored in the SQS queue is encrypted"
        id = "CKV_AWS_27"
        supported_resources = ('aws_sqs_queue',)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        sqs_managed_sse_enabled = conf.get("sqs_managed_sse_enabled")
        if sqs_managed_sse_enabled and isinstance(sqs_managed_sse_enabled, list):
            if sqs_managed_sse_enabled[0]:
                return CheckResult.PASSED
            return CheckResult.FAILED

        kms_master_key_id = conf.get("kms_master_key_id")
        if kms_master_key_id and isinstance(kms_master_key_id, list):
            if kms_master_key_id[0]:
                return CheckResult.PASSED
            return CheckResult.FAILED
        return CheckResult.FAILED


check = SQSQueueEncryption()



from checkov.serverless.checks.complete import *  # noqa
from checkov.serverless.checks.custom import *  # noqa
from checkov.serverless.checks.function import *  # noqa
from checkov.serverless.checks.layer import *  # noqa
from checkov.serverless.checks.package import *  # noqa
from checkov.serverless.checks.provider import *  # noqa
from checkov.serverless.checks.service import *  # noqa


from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.provider.base_check import BaseProviderCheck


class OpenstackCredentials(BaseProviderCheck):
    def __init__(self) -> None:
        name = "Ensure no hard coded OpenStack password, token, or application_credential_secret exists in provider"
        id = "CKV_OPENSTACK_1"
        supported_provider = ["openstack"]
        categories = [CheckCategories.SECRETS]
        super().__init__(name=name, id=id, categories=categories, supported_provider=supported_provider)

    def scan_provider_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """
        see: https://registry.terraform.io/providers/terraform-provider-openstack/openstack/latest/docs#configuration-reference
        """
        result = CheckResult.PASSED
        if conf.get("password"):
            conf[f'{self.id}_secret_1'] = conf.get('password')[0]
            result = CheckResult.FAILED
        if conf.get("token"):
            conf[f'{self.id}_secret_2'] = conf.get('token')[0]
            result = CheckResult.FAILED
        if conf.get("application_credential_secret"):
            conf[f'{self.id}_secret_3'] = conf.get('application_credential_secret')[0]
            result = CheckResult.FAILED
        return result


check = OpenstackCredentials()



from checkov.common.checks_infra.solvers.complex_solvers.or_solver import OrSolver  # noqa
from checkov.common.checks_infra.solvers.complex_solvers.and_solver import AndSolver  # noqa
from checkov.common.checks_infra.solvers.complex_solvers.not_solver import NotSolver  # noqa



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


def get_recursively(search_dict, field):
    """
    Takes a dict with nested lists and dicts,
    and searches all dicts for a key of the field
    provided.
    """
    fields_found = []

    for key, value in search_dict.items():

        if key == field:
            fields_found.append(value)

        elif isinstance(value, dict):
            results = get_recursively(value, field)
            for result in results:
                fields_found.append(result)

        elif isinstance(value, list):
            for item in value:
                if isinstance(item, dict):
                    more_results = get_recursively(item, field)
                    for another_result in more_results:
                        fields_found.append(another_result)

    return fields_found


class KMSKeyWildCardPrincipal(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure KMS key policy does not contain wildcard (*) principal"
        id = "CKV_AWS_33"
        supported_resources = ['AWS::KMS::Key']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/KeyPolicy/Statement/Principal'

    def scan_resource_conf(self, conf):
        if conf.get('Properties'):
            if conf['Properties'].get('KeyPolicy'):
                policy_block = conf['Properties']['KeyPolicy']
                principals_list = get_recursively(policy_block, 'Principal')
                for principal in principals_list:
                    if isinstance(principal, dict):
                        for principal_value in principal.values():
                            if principal_value == '*' or (isinstance(principal_value, list) and '*' in principal_value):
                                return CheckResult.FAILED
                    else:
                        if principal == '*':
                            return CheckResult.FAILED

        return CheckResult.PASSED


check = KMSKeyWildCardPrincipal()



from __future__ import annotations

import logging
import re
from collections import defaultdict
from pathlib import Path
from typing import TYPE_CHECKING, cast

from charset_normalizer import from_path

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import FileToPersist
    from checkov.common.typing import _SkippedCheck, _ScaSuppressions

CVE_COMMENT_PATTERN = re.compile(r"(checkov:skip=|bridgecrew:skip=) *(CVE-[\-\d]+)(:[^\n]+)?")
PACKAGE_COMMENT_PATTERN = re.compile(r"(checkov:skip=|bridgecrew:skip=) *([^\n]+)?")
PACKAGE_CVE_PATTERN = re.compile(r"(?P<package_name>.+)\[(?P<cves>[\w\-,]+)]")
SUPPORTED_SKIP_PACKAGE_FILES = {
    "requirements.txt",
}


def extract_inline_suppressions(uploaded_files: list[FileToPersist], rootless_file_path: str) -> _ScaSuppressions:
    """Extracts inline suppression

    ex. response
    suppressions = {
        "cve": {
            "CVE-2023-123": {
                "suppress_comment": "bla bla",
                "line_number": 1,
            }
        },
        "package": {
            "django": {
                "CVE-2023-456": {
                    "suppress_comment": "bla bla",
                    "line_number": 1,
                }
            }
            "flask": {
                "suppress_comment": "bla bla",
                "line_number": 1,
            }
        }
    }
    """

    full_file_path = next(
        (file.full_file_path for file in uploaded_files if file.s3_file_key == rootless_file_path), None
    )
    if not full_file_path:
        logging.debug(f"Package file {rootless_file_path} could not be found in uploaded files")
        return {}

    file_path = Path(full_file_path)
    if file_path.name not in PACKAGE_TYPE_EXTRACTOR:
        logging.debug(f"Package file type {file_path} not supported for skip comments")
        return {}

    try:
        content = file_path.read_text()
    except UnicodeDecodeError:
        logging.debug(f"Encoding for file {file_path} is not UTF-8, trying to detect it")
        content = str(from_path(file_path).best())
    except Exception:
        logging.debug(f"Error while trying to read file {file_path}", exc_info=True)
        return {}

    suppressions = PACKAGE_TYPE_EXTRACTOR[file_path.name](content)

    return suppressions


def extract_requirements_txt_suppressions(content: str) -> _ScaSuppressions:
    suppressions: _ScaSuppressions = defaultdict(dict)  # type:ignore[assignment]

    for line_number, line_text in enumerate(content.splitlines()):
        package_skip = re.search(PACKAGE_COMMENT_PATTERN, line_text)
        if package_skip:
            # check if it is just a CVE skip
            cve_skip = re.search(CVE_COMMENT_PATTERN, line_text)
            if cve_skip:
                suppression: _SkippedCheck = {
                    "suppress_comment": cve_skip.group(3)[1:] if cve_skip.group(3) else "No comment provided",
                    "line_number": line_number,
                }
                suppressions["cve"][cve_skip.group(2)] = suppression
            else:
                package_name = package_skip.group(2)
                suppress_comment = None
                if ":" in package_name:
                    # could result in incorrect splitting, if the package name has a colon
                    package_name, suppress_comment = package_name.rsplit(":", maxsplit=1)

                if "[" in package_name and "]" in package_name:
                    # specific CVEs should be suppressed for the package
                    package_cves = re.search(PACKAGE_CVE_PATTERN, package_name)
                    if package_cves:
                        package_cve_suppressions = {
                            package_cve: cast(
                                "_SkippedCheck",
                                {
                                    "suppress_comment": suppress_comment if suppress_comment else "No comment provided",
                                    "line_number": line_number,
                                },
                            )
                            for package_cve in package_cves.group("cves").split(",")
                        }
                        suppressions["package"][package_cves.group("package_name")] = package_cve_suppressions
                        continue

                suppression = {
                    "suppress_comment": suppress_comment if suppress_comment else "No comment provided",
                    "line_number": line_number,
                }
                suppressions["package"][package_name] = suppression

    return suppressions


PACKAGE_TYPE_EXTRACTOR = {
    "requirements.txt": extract_requirements_txt_suppressions,
}



from typing import List, Any, Dict

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.consts import ANY_VALUE


class SecretsEncrypted(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        #  -from github docs "It is also advised that you do not store plaintext values in your code but rather populate
        #  the encrypted_value using fields from a resource, data source or variable as,
        #  while encrypted in state, these will be easily accessible in your code"
        name = "Ensure GitHub Actions secrets are encrypted"
        id = "CKV_GIT_4"
        supported_resources = (
            "github_actions_environment_secret",
            "github_actions_organization_secret",
            "github_actions_secret",
        )
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        plaintext = conf.get("plaintext_value")
        if plaintext and self._is_variable_dependant(plaintext[0]):
            return CheckResult.UNKNOWN

        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "plaintext_value"

    def get_forbidden_values(self) -> List[Any]:
        return [ANY_VALUE]


check = SecretsEncrypted()



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class AllowPrivilegeEscalation(BaseK8sContainerCheck):
    def __init__(self) -> None:
        # CIS-1.3 1.7.5
        # CIS-1.5 5.2.5
        # https://kubernetes.io/docs/tasks/configure-pod-container/security-context/
        # https://kubernetes.io/docs/concepts/policy/pod-security-policy/
        # Default is allow / true
        # AllowPrivilegeEscalation is true always when the container is: 1) run as Privileged OR 2) has CAP_SYS_ADMIN.
        # This could be enforced via PodSecurityPolicy
        name = "Containers should not run with allowPrivilegeEscalation"
        id = "CKV_K8S_20"
        # Location: container .securityContext.allowPrivilegeEscalation
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["securityContext/allowPrivilegeEscalation"]
        if isinstance(conf, dict) and conf.get("securityContext"):
            if conf["securityContext"].get("allowPrivilegeEscalation") is False:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = AllowPrivilegeEscalation()



from checkov.common.vcs.vcs_schema import VCSSchema


class NoBranchProtectionSchema(VCSSchema):
    def __init__(self) -> None:
        schema = {
            "$schema": "http://json-schema.org/draft-04/schema#",
            "type": "object",
            "properties": {
                "message": {
                    "type": "string"
                },
                "documentation_url": {
                    "type": "string"
                }
            },
            "required": [
                "message",
                "documentation_url"
            ]
        }
        super().__init__(schema=schema)


schema = NoBranchProtectionSchema()



from __future__ import annotations

import logging
from enum import Enum
from typing import Dict, Any

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import Report

from checkov.common.runners.base_post_runner import BasePostRunner
from checkov.common.util.type_forcers import force_list
from checkov.policies_3d.record import Policy3dRecord
from checkov.policies_3d.checks_infra.base_check import Base3dPolicyCheck
from checkov.runner_filter import RunnerFilter


class CVECheckAttribute(str, Enum):
    RISK_FACTORS = "risk_factors"


class CVEReportAttribute(str, Enum):
    RISK_FACTORS = 'riskFactors'


CVE_CHECK_TO_REPORT_ATTRIBUTE = {
    CVECheckAttribute.RISK_FACTORS: CVEReportAttribute.RISK_FACTORS
}


class Policy3dRunner(BasePostRunner):
    check_type = CheckType.POLICY_3D  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__()

    def run(  # type:ignore[override]
            self,
            checks: list[Base3dPolicyCheck] | None = None,
            scan_reports: list[Report] | None = None,
            runner_filter: RunnerFilter | None = None
    ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)

        if not checks or not scan_reports:
            logging.debug("No checks or reports scan.")
            return report

        self.pbar.initiate(len(checks))

        reports_by_framework = {report.check_type: report for report in scan_reports}
        for check in checks:
            records = self.collect_check(check, reports_by_framework)
            for record in records:
                report.add_record(record=record)

        self.pbar.close()
        return report

    def collect_check(self, check: Base3dPolicyCheck, reports_by_fw: dict[str, Report]) -> list[Policy3dRecord]:
        records = []
        iac_results_map = self.solve_check_iac(check, reports_by_fw)
        cve_results_map = self.solve_check_cve(check, reports_by_fw)

        for iac_resource, iac_records in iac_results_map.items():
            for cve_resource, vulnerabilities in cve_results_map.items():
                if iac_resource == cve_resource:
                    # This means we found the combination on the same resource -> create a violation for that resource
                    check_result = CheckResult.FAILED
                    iac_records = [record for record in iac_records]
                    record = self.get_record(check, iac_records[0], vulnerabilities, check_result, iac_records)
                    record.set_guideline(check.guideline)
                    records.append(record)

        self.pbar.update()
        return records

    def solve_check_iac(self, check: Base3dPolicyCheck, reports_by_fw: dict[str, Report]) -> dict[str, list[Record]]:
        iac_results_map: dict[str, list[Record]] = {}
        if check.iac:
            for framework, bc_check_ids in check.iac.items():
                framework_report = reports_by_fw.get(framework)
                if framework_report:
                    fw_records = framework_report.failed_checks
                    for record in fw_records:
                        if record.bc_check_id in bc_check_ids:
                            resource_id = f'{record.file_abs_path}:{record.resource}'
                            if resource_id in iac_results_map:
                                iac_results_map[resource_id].append(record)
                            else:
                                iac_results_map[resource_id] = [record]

                    # the following implements the AND logic for the list of iac expected check ids
                    for resource in list(iac_results_map.keys()):
                        if len(iac_results_map[resource]) != len(bc_check_ids):
                            del iac_results_map[resource]

        return iac_results_map

    def solve_check_cve(self, check: Base3dPolicyCheck, reports_by_fw: dict[str, Report]) -> dict[str, list[Dict[str, Any]]]:
        cve_results_map: dict[str, list[Dict[str, Any]]] = {}
        if check.cve:
            cve_report = reports_by_fw.get(CheckType.SCA_IMAGE)
            if cve_report:
                image_results = cve_report.image_cached_results
                for attribute, value in check.cve.items():
                    for image_result in image_results:
                        matching_cves = [vuln for vuln in image_result.get('vulnerabilities', []) if
                                         value[0] in force_list(vuln.get(CVE_CHECK_TO_REPORT_ATTRIBUTE[attribute], []))]
                        if matching_cves:
                            image_related_resource = image_result.get('relatedResourceId')
                            image_name = image_result.get('dockerImageName')
                            if not image_related_resource or not image_name:
                                logging.debug(
                                    "[policies3d/runner](solve_check_cve) Found vulnerabilities of an image without a related resource or image name, skipping")
                                break
                            for cve in matching_cves:
                                cve['dockerImageName'] = image_name
                            # The current logic for multiple cve conditions in the policy is of "OR" - we add all of
                            # the matching cves, even if matched only by a single policy attribute. To implement an
                            # "AND" logic for the combination of conditions, the matching cves need to be filtered
                            # before being added to the result.
                            if image_related_resource in cve_results_map:
                                cve_results_map[image_related_resource].extend(matching_cves)
                            else:
                                cve_results_map[image_related_resource] = matching_cves
        return cve_results_map

    def get_record(self, check: Base3dPolicyCheck, iac_record: Record, vulnerabilities: list[Dict[str, Any]],
                   check_result: CheckResult, iac_records: list[Record]) -> Policy3dRecord:
        return Policy3dRecord(
            check_id=check.id,
            bc_check_id=check.bc_id,
            check_name=check.name,
            check_result={'result': check_result},
            code_block=iac_record.code_block,
            file_path=iac_record.file_path,
            file_line_range=iac_record.file_line_range,
            resource=f'{iac_record.file_path}:{iac_record.resource}',
            evaluations=None,
            check_class=check.__class__.__module__,
            file_abs_path=iac_record.file_abs_path,
            severity=check.severity,
            vulnerabilities=vulnerabilities,
            iac_records=iac_records
        )



class ResourceKeywordIdentifier:
    """
    this class maps connections between resources by their unique keyword identifier.
    each resource in this class has a list of objects, and each object defines a potential connection to a different
    resource only if all attributes in the object are matched. each object in the list is independent regardless of
    the other objects in the list.

    for example:
    A ServiceAccount resource A with the property 'metadata.name' with value 'service-123' will match a resource B
    of type 'ClusterRoleBinding' with a 'subjects.name' property equals to 'service-123'
    and
    the property 'kind' for resource A matches resource B's 'subjects.kind' property
    """

    KINDS_KEYWORDS_MAP = {
        # TODO: "PersistentVolumeClaim": ["claimName"],
        # TODO: "ClusterRole": ["rules[].resources", "rules[].resourceNames"],
        # TODO: "Role": ["rules[].resources", "rules[].resourceNames"],

        "ServiceAccount": [
            {"spec.serviceAccountName": "metadata.name"}
        ],
        "ClusterRoleBinding": [
            {"metadata.name": "roleRef.name", "kind": "roleRef.kind"},
            [{"subjects": {"metadata.name": "name", "kind": "kind"}}]
        ],
        "RoleBinding": [
            {"metadata.name": "roleRef.name", "kind": "roleRef.kind"},
            [{"subjects": {"metadata.name": "name", "kind": "kind", "metadata.namespace": "namespace"}}]
        ]
    }



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Union


from checkov.common.util.data_structures_utils import find_in_dict
from checkov.terraform.image_referencer.base_provider import BaseTerraformProvider

if TYPE_CHECKING:
    from networkx import DiGraph
    import igraph


class GcpTerraformProvider(BaseTerraformProvider):
    def __init__(self, graph_connector: Union[igraph.Graph, DiGraph]) -> None:
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=SUPPORTED_GCP_IMAGE_RESOURCE_TYPES,
        )


def extract_images_from_google_cloudbuild_trigger(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    steps = find_in_dict(input_dict=resource, key_path="build/step")
    if isinstance(steps, list):
        for definition in steps:
            name = definition.get("name")
            if name and isinstance(name, str):
                image_names.append(name)

    return image_names


def extract_images_from_google_cloud_run_service(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    name = find_in_dict(input_dict=resource, key_path="template/spec/containers/image")
    if name and isinstance(name, str):
        image_names.append(name)

    return image_names


def extract_images_from_google_cloud_run_v2_job(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    name = find_in_dict(input_dict=resource, key_path="template/template/containers/image")
    if name and isinstance(name, str):
        image_names.append(name)

    return image_names


def extract_images_from_google_cloud_run_v2_service(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    name = find_in_dict(input_dict=resource, key_path="template/containers/image")
    if name and isinstance(name, str):
        image_names.append(name)

    return image_names


# needs to be at the bottom to add the defined functions
SUPPORTED_GCP_IMAGE_RESOURCE_TYPES = {
    "google_cloudbuild_trigger": extract_images_from_google_cloudbuild_trigger,
    "google_cloud_run_service": extract_images_from_google_cloud_run_service,
    "google_cloud_run_v2_job": extract_images_from_google_cloud_run_v2_job,
    "google_cloud_run_v2_service": extract_images_from_google_cloud_run_v2_service,
}



from __future__ import annotations

import json
import logging
import os.path
import re
import uuid
import webbrowser
from collections import namedtuple
from concurrent import futures
from io import StringIO
from json import JSONDecodeError
from os import path
from pathlib import Path
from time import sleep
from typing import List, Dict, TYPE_CHECKING, Any, cast

import boto3  # type:ignore[import]
import dpath.util
import requests
import urllib3
from botocore.exceptions import ClientError  # type:ignore[import]
from botocore.config import Config  # type:ignore[import]
from cachetools import cached, TTLCache
from colorama import Style
from termcolor import colored
from tqdm import trange
from urllib3.exceptions import HTTPError, MaxRetryError

from checkov.common.bridgecrew.run_metadata.registry import registry
from checkov.common.bridgecrew.platform_errors import BridgecrewAuthError
from checkov.common.bridgecrew.platform_key import read_key, persist_key, bridgecrew_file
from checkov.common.bridgecrew.wrapper import reduce_scan_reports, persist_checks_results, \
    enrich_and_persist_checks_metadata, checkov_results_prefix, persist_run_metadata, _put_json_object, \
    persist_logs_stream
from checkov.common.models.consts import SUPPORTED_FILE_EXTENSIONS, SUPPORTED_FILES, SCANNABLE_PACKAGE_FILES
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.base_runner import filter_ignored_paths
from checkov.common.typing import _CicdDetails
from checkov.common.util.consts import PRISMA_PLATFORM, BRIDGECREW_PLATFORM, CHECKOV_RUN_SCA_PACKAGE_SCAN_V2
from checkov.common.util.data_structures_utils import merge_dicts
from checkov.common.util.http_utils import normalize_prisma_url, get_auth_header, get_default_get_headers, \
    get_user_agent_header, get_default_post_headers, get_prisma_get_headers, get_prisma_auth_header, \
    get_auth_error_message, normalize_bc_url
from checkov.common.util.type_forcers import convert_prisma_policy_filter_to_dict, convert_str_to_bool
from checkov.secrets.coordinator import EnrichedSecret
from checkov.version import version as checkov_version

if TYPE_CHECKING:
    import argparse
    from botocore.client import BaseClient  # type:ignore[import]
    from checkov.common.bridgecrew.bc_source import SourceType
    from checkov.common.output.report import Report
    from requests import Response
    from typing_extensions import TypeGuard


SLEEP_SECONDS = 1

EMAIL_PATTERN = re.compile(r"[^@]+@[^@]+\.[^@]+")
UUID_V4_PATTERN = re.compile(r"^[0-9a-f]{8}\b-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-\b[0-9a-f]{12}$")
# found at https://regexland.com/base64/
BASE64_PATTERN = re.compile(r"^(?:[A-Za-z\d+/]{4})*(?:[A-Za-z\d+/]{3}=|[A-Za-z\d+/]{2}==)?$")
REPO_PATH_PATTERN = re.compile(r'checkov/(.*?)/src')

ACCOUNT_CREATION_TIME = 180  # in seconds

UNAUTHORIZED_MESSAGE = 'User is not authorized to access this resource with an explicit deny'
ASSUME_ROLE_UNUATHORIZED_MESSAGE = 'is not authorized to perform: sts:AssumeRole'

FileToPersist = namedtuple('FileToPersist', 'full_file_path s3_file_key')

DEFAULT_REGION = "us-west-2"
GOV_CLOUD_REGION = 'us-gov-west-1'
PRISMA_GOV_API_URL = 'https://api.gov.prismacloud.io'
MAX_RETRIES = 40
ONBOARDING_SOURCE = "checkov"

SIGNUP_HEADER = merge_dicts({
    'Accept': 'application/json',
    'Content-Type': 'application/json;charset=UTF-8'},
    get_user_agent_header())
CI_METADATA_EXTRACTOR = registry.get_extractor()


class BcPlatformIntegration:
    def __init__(self) -> None:
        self.bc_api_key = read_key()
        self.s3_client: BaseClient | None = None
        self.bucket: str | None = None
        self.credentials: dict[str, str] | None = None
        self.repo_path: str | None = None
        self.repo_id: str | None = None
        self.repo_branch: str | None = None
        self.skip_fixes = False
        self.skip_download = False
        self.bc_source: SourceType | None = None
        self.bc_source_version: str | None = None
        self.timestamp: str | None = None
        self.scan_reports: list[Report] = []
        self.bc_api_url = normalize_bc_url(os.getenv('BC_API_URL', "https://www.bridgecrew.cloud"))
        self.prisma_api_url = normalize_prisma_url(os.getenv("PRISMA_API_URL"))
        self.prisma_policies_url: str | None = None
        self.prisma_policy_filters_url: str | None = None
        self.setup_api_urls()
        self.customer_run_config_response = None
        self.prisma_policies_response = None
        self.public_metadata_response = None
        self.use_s3_integration = False
        self.platform_integration_configured = False
        self.http: urllib3.PoolManager | urllib3.ProxyManager | None = None
        self.bc_skip_mapping = False
        self.cicd_details: _CicdDetails = {}

    def set_bc_api_url(self, new_url: str) -> None:
        self.bc_api_url = normalize_bc_url(new_url)
        self.setup_api_urls()

    def setup_api_urls(self) -> None:
        """
        API URLs vary depending upon whether the platform is Bridgecrew or Prisma Cloud.
        Bridgecrew has one default that can be used when initializing the class,
        but Prisma Cloud requires resetting them in setup_bridgecrew_credentials,
        which is where command-line parameters are first made available.
        """
        if self.prisma_api_url:
            self.api_url = f"{self.prisma_api_url}/bridgecrew"
            self.prisma_policies_url = f"{self.prisma_api_url}/v2/policy"
            self.prisma_policy_filters_url = f"{self.prisma_api_url}/filter/policy/suggest"
        else:
            self.api_url = self.bc_api_url
        self.guidelines_api_url = f"{self.api_url}/api/v2/guidelines"
        self.guidelines_api_url_backoff = f"{self.api_url}/api/v1/guidelines"

        self.integrations_api_url = f"{self.api_url}/api/v1/integrations/types/checkov"
        self.onboarding_url = f"{self.api_url}/api/v1/signup/checkov"
        self.platform_run_config_url = f"{self.api_url}/api/v2/checkov/runConfiguration"
        self.platform_run_config_url_backoff = f"{self.api_url}/api/v1/checkov/runConfiguration"

    def is_prisma_integration(self) -> bool:
        if self.bc_api_key and not self.is_bc_token(self.bc_api_key):
            return True
        return False

    @staticmethod
    def is_token_valid(token: str) -> bool:
        parts = token.split('::')
        parts_len = len(parts)
        if parts_len == 1:
            return BcPlatformIntegration.is_bc_token(token)
        elif parts_len == 2:
            # A Prisma access key is a UUID, same as a BC API key
            if BcPlatformIntegration.is_bc_token(parts[0]) and parts[1] and BASE64_PATTERN.match(parts[1]) is not None:
                return True
            return False
        else:
            return False

    @staticmethod
    def is_bc_token(token: str | None) -> TypeGuard[str]:
        if not token:
            return False

        return re.match(UUID_V4_PATTERN, token) is not None

    @cached(TTLCache(maxsize=1, ttl=540))
    def get_auth_token(self) -> str:
        if self.is_bc_token(self.bc_api_key):
            return self.bc_api_key
        # A Prisma Cloud Access Key was specified as the Bridgecrew token.
        if not self.prisma_api_url:
            raise ValueError("A Prisma Cloud token was set, but no Prisma Cloud API URL was set")
        if not self.bc_api_key:
            # should usually not happen
            raise ValueError("A Prisma Cloud or Birdgecrew token was not set")
        if '::' not in self.bc_api_key:
            raise ValueError(
                "A Prisma Cloud token was set, but the token is not in the correct format: <access_key_id>::<secret_key>")
        if not self.http:
            raise AttributeError("HTTP manager was not correctly created")
        username, password = self.bc_api_key.split('::')
        request = self.http.request("POST", f"{self.prisma_api_url}/login",  # type:ignore[no-untyped-call]
                                    body=json.dumps({"username": username, "password": password}),
                                    headers=merge_dicts({"Content-Type": "application/json"}, get_user_agent_header()))
        if request.status == 401:
            logging.error(f'Received 401 response from Prisma /login endpoint: {request.data.decode("utf8")}')
            raise BridgecrewAuthError()
        elif request.status == 403:
            logging.error('Received 403 (Forbidden) response from Prisma /login endpoint')
            raise BridgecrewAuthError()
        token: str = json.loads(request.data.decode("utf8"))['token']
        return token

    def setup_http_manager(self, ca_certificate: str | None = None) -> None:
        """
        bridgecrew uses both the urllib3 and requests libraries, while checkov uses the requests library.
        :param ca_certificate: an optional CA bundle to be used by both libraries.
        """
        ca_certificate = ca_certificate or os.getenv('BC_CA_BUNDLE')

        if self.http:
            return
        if ca_certificate:
            os.environ['REQUESTS_CA_BUNDLE'] = ca_certificate
            try:
                parsed_url = urllib3.util.parse_url(os.environ['https_proxy'])
                self.http = urllib3.ProxyManager(os.environ['https_proxy'], cert_reqs='REQUIRED',
                                                 ca_certs=ca_certificate,
                                                 proxy_headers=urllib3.make_headers(proxy_basic_auth=parsed_url.auth))  # type:ignore[no-untyped-call]
            except KeyError:
                self.http = urllib3.PoolManager(cert_reqs='REQUIRED', ca_certs=ca_certificate)
        else:
            try:
                parsed_url = urllib3.util.parse_url(os.environ['https_proxy'])
                self.http = urllib3.ProxyManager(os.environ['https_proxy'],
                                                 proxy_headers=urllib3.make_headers(proxy_basic_auth=parsed_url.auth))  # type:ignore[no-untyped-call]
            except KeyError:
                self.http = urllib3.PoolManager()

    def setup_bridgecrew_credentials(
        self,
        repo_id: str,
        skip_fixes: bool = False,
        skip_download: bool = False,
        source: SourceType | None = None,
        source_version: str | None = None,
        repo_branch: str | None = None,
        prisma_api_url: str | None = None,
    ) -> None:
        """
        Setup credentials against Bridgecrew's platform.
        :param repo_id: Identity string of the scanned repository, of the form <repo_owner>/<repo_name>
        :param skip_fixes: whether to skip querying fixes from Bridgecrew
        :param skip_download: whether to skip downloading data (guidelines, custom policies, etc) from the platform
        :param source:
        :param prisma_api_url: optional URL for the Prisma Cloud platform, requires a Prisma Cloud Access Key as bc_api_key
        """
        self.repo_id = repo_id
        self.repo_branch = repo_branch
        self.skip_fixes = skip_fixes
        self.skip_download = skip_download
        self.bc_source = source
        self.bc_source_version = source_version

        if prisma_api_url:
            self.prisma_api_url = normalize_prisma_url(prisma_api_url)
            self.setup_api_urls()
            logging.info(f'Using Prisma API URL: {self.prisma_api_url}')

        if self.bc_source and self.bc_source.upload_results:
            self.set_s3_integration()

        self.platform_integration_configured = True

    def set_s3_integration(self) -> None:
        region = DEFAULT_REGION
        use_accelerate_endpoint = True
        if self.prisma_api_url == PRISMA_GOV_API_URL:
            region = GOV_CLOUD_REGION
            use_accelerate_endpoint = False

        try:
            self.skip_fixes = True  # no need to run fixes on CI integration
            repo_full_path, response = self.get_s3_role(self.repo_id)  # type: ignore
            self.bucket, self.repo_path = repo_full_path.split("/", 1)
            self.timestamp = self.repo_path.split("/")[-2]
            self.credentials = cast("dict[str, str]", response["creds"])
            config = Config(
                s3={
                    "use_accelerate_endpoint": use_accelerate_endpoint,
                }
            )
            self.s3_client = boto3.client(
                "s3",
                aws_access_key_id=self.credentials["AccessKeyId"],
                aws_secret_access_key=self.credentials["SecretAccessKey"],
                aws_session_token=self.credentials["SessionToken"],
                region_name=region,
                config=config,
            )
            self.use_s3_integration = True
        except MaxRetryError:
            logging.error("An SSL error occurred connecting to the platform. If you are on a VPN, please try "
                          "disabling it and re-running the command.", exc_info=True)
            raise
        except HTTPError:
            logging.error("Failed to get customer assumed role", exc_info=True)
            raise
        except ClientError:
            logging.error(f"Failed to initiate client with credentials {self.credentials}", exc_info=True)
            raise
        except JSONDecodeError:
            logging.error(f"Response of {self.integrations_api_url} is not a valid JSON", exc_info=True)
            raise
        except BridgecrewAuthError:
            logging.error("Received an error response during authentication")
            raise

    def get_s3_role(self, repo_id: str) -> tuple[str, dict[str, Any]]:
        token = self.get_auth_token()

        if not self.http:
            raise AttributeError("HTTP manager was not correctly created")

        request = self.http.request("POST", self.integrations_api_url, body=json.dumps({"repoId": repo_id}),  # type:ignore[no-untyped-call]
                                    headers=merge_dicts({"Authorization": token, "Content-Type": "application/json"},
                                                        get_user_agent_header()))
        if request.status == 403:
            error_message = get_auth_error_message(request.status, self.is_prisma_integration(), True)
            raise BridgecrewAuthError(error_message)
        response = json.loads(request.data.decode("utf8"))
        while ('Message' in response or 'message' in response):
            if 'Message' in response and response['Message'] == UNAUTHORIZED_MESSAGE:
                raise BridgecrewAuthError()
            if 'message' in response and ASSUME_ROLE_UNUATHORIZED_MESSAGE in response['message']:
                raise BridgecrewAuthError(
                    "Checkov got an unexpected authorization error that may not be due to your credentials. Please contact support.")
            if 'message' in response and "cannot be found" in response['message']:
                self.loading_output("creating role")
                request = self.http.request("POST", self.integrations_api_url, body=json.dumps({"repoId": repo_id}),  # type:ignore[no-untyped-call]
                                            headers=merge_dicts(
                                                {"Authorization": token, "Content-Type": "application/json"},
                                                get_user_agent_header()))
                response = json.loads(request.data.decode("utf8"))

        repo_full_path = response["path"]
        return repo_full_path, response

    def is_integration_configured(self) -> bool:
        """
        Checks if Bridgecrew integration is fully configured based in input params.
        :return: True if the integration is configured, False otherwise
        """
        return self.platform_integration_configured

    def persist_repository(
        self,
        root_dir: str | Path,
        files: list[str] | None = None,
        excluded_paths: list[str] | None = None,
        included_paths: list[str] | None = None,
    ) -> None:
        """
        Persist the repository found on root_dir path to Bridgecrew's platform. If --file flag is used, only files
        that are specified will be persisted.
        :param files: Absolute path of the files passed in the --file flag.
        :param root_dir: Absolute path of the directory containing the repository root level.
        :param excluded_paths: Paths to exclude from persist process
        :param included_paths: Paths to exclude from persist process
        """
        excluded_paths = excluded_paths if excluded_paths is not None else []

        if not self.use_s3_integration:
            return
        files_to_persist: List[FileToPersist] = []
        if files:
            for f in files:
                f_name = os.path.basename(f)
                _, file_extension = os.path.splitext(f)
                if CHECKOV_RUN_SCA_PACKAGE_SCAN_V2 and file_extension in SCANNABLE_PACKAGE_FILES:
                    continue
                if file_extension in SUPPORTED_FILE_EXTENSIONS or f_name in SUPPORTED_FILES:
                    files_to_persist.append(FileToPersist(f, os.path.relpath(f, root_dir)))
        else:
            for root_path, d_names, f_names in os.walk(root_dir):
                # self.excluded_paths only contains the config fetched from the platform.
                # but here we expect the list from runner_registry as well (which includes self.excluded_paths).
                filter_ignored_paths(root_path, d_names, excluded_paths, included_paths=included_paths)
                filter_ignored_paths(root_path, f_names, excluded_paths)
                for file_path in f_names:
                    _, file_extension = os.path.splitext(file_path)
                    if CHECKOV_RUN_SCA_PACKAGE_SCAN_V2 and file_extension in SCANNABLE_PACKAGE_FILES:
                        continue
                    if file_extension in SUPPORTED_FILE_EXTENSIONS or file_path in SUPPORTED_FILES:
                        full_file_path = os.path.join(root_path, file_path)
                        relative_file_path = os.path.relpath(full_file_path, root_dir)
                        files_to_persist.append(FileToPersist(full_file_path, relative_file_path))

        self.persist_files(files_to_persist)

    def persist_git_configuration(self, root_dir: str | Path, git_config_folders: list[str]) -> None:
        if not self.use_s3_integration:
            return
        files_to_persist: list[FileToPersist] = []

        for git_config_folder in git_config_folders:
            if not os.path.isdir(git_config_folder):
                continue
            if not len(os.listdir(git_config_folder)):
                continue

            for root_path, _, f_names in os.walk(git_config_folder):
                for file_path in f_names:
                    _, file_extension = os.path.splitext(file_path)
                    if file_extension in SUPPORTED_FILE_EXTENSIONS:
                        full_file_path = os.path.join(root_path, file_path)
                        relative_file_path = os.path.relpath(full_file_path, root_dir)
                        files_to_persist.append(FileToPersist(full_file_path, relative_file_path))

        self.persist_files(files_to_persist)

    def persist_scan_results(self, scan_reports: list[Report]) -> None:
        """
        Persist checkov's scan result into bridgecrew's platform.
        :param scan_reports: List of checkov scan reports
        """
        if not self.use_s3_integration:
            return
        if not self.bucket or not self.repo_path:
            logging.error(f"Something went wrong: bucket {self.bucket}, repo path {self.repo_path}")
            return

        # just process reports with actual results in it
        self.scan_reports = [scan_report for scan_report in scan_reports if not scan_report.is_empty(full=True)]

        reduced_scan_reports = reduce_scan_reports(self.scan_reports)
        checks_metadata_paths = enrich_and_persist_checks_metadata(self.scan_reports, self.s3_client, self.bucket,
                                                                   self.repo_path)
        dpath.util.merge(reduced_scan_reports, checks_metadata_paths)
        persist_checks_results(reduced_scan_reports, self.s3_client, self.bucket, self.repo_path)

    def persist_image_scan_results(self, report: dict[str, Any] | None, file_path: str, image_name: str, branch: str) -> None:
        if not self.bucket or not self.repo_path:
            logging.error("Bucket or repo_path was not set")
            return

        repo_path_without_src = os.path.dirname(self.repo_path)
        target_report_path = f'{repo_path_without_src}/{checkov_results_prefix}/{CheckType.SCA_IMAGE}/raw_results.json'
        to_upload = {"report": report, "file_path": file_path, "image_name": image_name, "branch": branch}
        _put_json_object(self.s3_client, to_upload, self.bucket, target_report_path)

    def persist_enriched_secrets(self, enriched_secrets: list[EnrichedSecret]) -> str | None:
        if not enriched_secrets or not self.repo_path or not self.bucket:
            logging.debug(f'One of enriched secrets, repo path, or bucket are empty, aborting. values:'
                          f'enriched_secrets={"Valid" if enriched_secrets else "Empty"},'
                          f' repo_path={self.repo_path}, bucket={self.bucket}')
            return None

        if not bc_integration.bc_api_key or not os.getenv("CKV_VALIDATE_SECRETS"):
            logging.debug('Skipping persistence of enriched secrets object as secrets verification is off,'
                          ' enabled it via env var CKV_VALIDATE_SECRETS and provide an api key')
            return None

        base_path = re.sub(REPO_PATH_PATTERN, r'original_secrets/\1', self.repo_path)
        s3_path = f'{base_path}/{uuid.uuid4()}.json'
        try:
            _put_json_object(self.s3_client, enriched_secrets, self.bucket, s3_path)
        except ClientError:
            logging.warning("Got access denied, retrying as s3 role changes should be propagated")
            sleep(4)
            try:
                _put_json_object(self.s3_client, enriched_secrets, self.bucket, s3_path)
            except ClientError:
                logging.error("Getting access denied consistently, aborting secrets verification")
                return None

        return s3_path

    def persist_run_metadata(self, run_metadata: dict[str, str | list[str]]) -> None:
        if not self.use_s3_integration:
            return
        if not self.bucket or not self.repo_path:
            logging.error(f"Something went wrong: bucket {self.bucket}, repo path {self.repo_path}")
            return
        persist_run_metadata(run_metadata, self.s3_client, self.bucket, self.repo_path)

    def persist_logs_stream(self, logs_stream: StringIO) -> None:
        if not self.use_s3_integration:
            return
        if not self.bucket or not self.repo_path:
            logging.error(f"Something went wrong: bucket {self.bucket}, repo path {self.repo_path}")
            return
        persist_logs_stream(logs_stream, self.s3_client, self.bucket, self.repo_path)

    def commit_repository(self, branch: str) -> str | None:
        """
        :param branch: branch to be persisted
        Finalize the repository's scanning in bridgecrew's platform.
        """
        try_num = 0
        while try_num < MAX_RETRIES:
            if not self.use_s3_integration:
                return None

            request = None
            response = None
            try:
                if not self.http:
                    logging.error("HTTP manager was not correctly created")
                    return None
                if not self.bc_source:
                    logging.error("Source was not set")
                    return None
                if not self.bc_source.upload_results:
                    # no need to upload something
                    return None

                request = self.http.request("PUT", f"{self.integrations_api_url}?source={self.bc_source.name}",  # type:ignore[no-untyped-call]
                                            body=json.dumps(
                                                {"path": self.repo_path, "branch": branch,
                                                 "to_branch": CI_METADATA_EXTRACTOR.to_branch,
                                                 "pr_id": CI_METADATA_EXTRACTOR.pr_id,
                                                 "pr_url": CI_METADATA_EXTRACTOR.pr_url,
                                                 "commit_hash": CI_METADATA_EXTRACTOR.commit_hash,
                                                 "commit_url": CI_METADATA_EXTRACTOR.commit_url,
                                                 "author": CI_METADATA_EXTRACTOR.author_name,
                                                 "author_url": CI_METADATA_EXTRACTOR.author_url,
                                                 "run_id": CI_METADATA_EXTRACTOR.run_id,
                                                 "run_url": CI_METADATA_EXTRACTOR.run_url,
                                                 "repository_url": CI_METADATA_EXTRACTOR.repository_url}),
                                            headers=merge_dicts({"Authorization": self.get_auth_token(),
                                                                 "Content-Type": "application/json",
                                                                 'x-api-client': self.bc_source.name,
                                                                 'x-api-checkov-version': checkov_version},
                                                                get_user_agent_header()
                                                                ))
                response = json.loads(request.data.decode("utf8"))
                url: str = response.get("url", None)
                return url
            except HTTPError:
                logging.error(f"Failed to commit repository {self.repo_path}", exc_info=True)
                raise
            except JSONDecodeError:
                if request:
                    logging.warning(f"Response (status: {request.status}) of {self.integrations_api_url}: {request.data.decode('utf8')}")
                logging.error(f"Response of {self.integrations_api_url} is not a valid JSON", exc_info=True)
                raise
            finally:
                if request and request.status == 201 and response and response.get("result") == "Success":
                    logging.info(f"Finalize repository {self.repo_id} in bridgecrew's platform")
                elif (
                    response
                    and try_num < MAX_RETRIES
                    and re.match("The integration ID .* in progress", response.get("message", ""))
                ):
                    logging.info(
                        f"Failed to persist for repo {self.repo_id}, sleeping for {SLEEP_SECONDS} seconds before retrying")
                    try_num += 1
                    sleep(SLEEP_SECONDS)
                else:
                    raise Exception(
                        f"Failed to finalize repository {self.repo_id} in bridgecrew's platform\n{response}")

        return None

    def persist_files(self, files_to_persist: list[FileToPersist]) -> None:
        logging.info(f"Persisting {len(files_to_persist)} files")
        with futures.ThreadPoolExecutor() as executor:
            futures.wait(
                [executor.submit(self._persist_file, file_to_persist.full_file_path, file_to_persist.s3_file_key) for
                 file_to_persist in files_to_persist],
                return_when=futures.FIRST_EXCEPTION,
            )
        logging.info(f"Done persisting {len(files_to_persist)} files")

    def _persist_file(self, full_file_path: str, s3_file_key: str) -> None:
        tries = MAX_RETRIES
        curr_try = 0

        if not self.s3_client or not self.bucket or not self.repo_path:
            logging.error(
                f"Something went wrong: S3 client {self.s3_client} bucket {self.bucket}, repo path {self.repo_path}"
            )
            return

        file_object_key = os.path.join(self.repo_path, s3_file_key).replace("\\", "/")
        while curr_try < tries:
            try:
                self.s3_client.upload_file(full_file_path, self.bucket, file_object_key)
                return
            except ClientError as e:
                if e.response.get('Error', {}).get('Code') == 'AccessDenied':
                    sleep(SLEEP_SECONDS)
                    curr_try += 1
                else:
                    logging.error(f"failed to persist file {full_file_path} into S3 bucket {self.bucket}",
                                  exc_info=True)
                    raise
            except Exception:
                logging.error(f"failed to persist file {full_file_path} into S3 bucket {self.bucket}", exc_info=True)
                raise
        if curr_try == tries:
            logging.error(
                f"failed to persist file {full_file_path} into S3 bucket {self.bucket} - gut AccessDenied {tries} times")

    def get_platform_run_config(self) -> None:
        if self.skip_download is True:
            logging.debug("Skipping downloading configs from platform")
            return

        if self.is_integration_configured():
            self.get_customer_run_config()
        else:
            self.get_public_run_config()

    def _get_run_config_query_params(self) -> str:
        return f'module={"bc" if self.is_bc_token(self.bc_api_key) else "pc"}&enforcementv2=true'

    def get_run_config_url(self) -> str:
        return f'{self.platform_run_config_url}?{self._get_run_config_query_params()}'

    def get_run_config_url_backoff(self) -> str:
        return f'{self.platform_run_config_url_backoff}?{self._get_run_config_query_params()}'

    def get_customer_run_config(self) -> None:
        if self.skip_download is True:
            logging.debug("Skipping customer run config API call")
            return

        if not self.bc_api_key or not self.is_integration_configured():
            raise Exception(
                "Tried to get customer run config, but the API key was missing or the integration was not set up")

        if not self.bc_source:
            logging.error("Source was not set")
            return

        try:
            token = self.get_auth_token()
            headers = merge_dicts(get_auth_header(token),
                                  get_default_get_headers(self.bc_source, self.bc_source_version))

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return

            platform_type = PRISMA_PLATFORM if self.is_prisma_integration() else BRIDGECREW_PLATFORM

            url = self.get_run_config_url()
            logging.debug(f'Platform run config URL: {url}')
            request = self.http.request("GET", url, headers=headers)  # type:ignore[no-untyped-call]
            if request.status != 200:
                url = self.get_run_config_url_backoff()
                logging.debug(f'Platform run config URL: {url}')
                request = self.http.request("GET", url, headers=headers)  # type:ignore[no-untyped-call]
                if request.status != 200:
                    error_message = get_auth_error_message(request.status, self.is_prisma_integration(), False)
                    logging.error(error_message)
                    raise BridgecrewAuthError(error_message)
            self.customer_run_config_response = json.loads(request.data.decode("utf8"))

            logging.debug(f"Got customer run config from {platform_type} platform")
        except Exception:
            logging.warning(f"Failed to get the customer run config from {self.platform_run_config_url}", exc_info=True)
            raise

    def get_prisma_build_policies(self, policy_filter: str) -> None:
        """
        Get Prisma policy for enriching runConfig with metadata
        Filters: https://prisma.pan.dev/api/cloud/cspm/policy#operation/get-policy-filters-and-options
        :param policy_filter: comma separated filter string. Example, policy.label=A,cloud.type=aws
        :return:
        """
        if self.skip_download is True:
            logging.debug("Skipping prisma policy API call")
            return
        if not policy_filter:
            return
        if not self.is_prisma_integration():
            return
        if not self.bc_api_key or not self.is_integration_configured():
            raise Exception(
                "Tried to get prisma build policy metadata, "
                "but the API key was missing or the integration was not set up")
        try:
            token = self.get_auth_token()
            headers = merge_dicts(get_prisma_auth_header(token), get_prisma_get_headers())

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return

            logging.debug(f'Prisma policy URL: {self.prisma_policies_url}')
            query_params = convert_prisma_policy_filter_to_dict(policy_filter)
            if self.is_valid_policy_filter(query_params, valid_filters=self.get_prisma_policy_filters()):
                # If enabled and subtype are not explicitly set, use the only acceptable values.
                query_params['policy.enabled'] = True
                query_params['policy.subtype'] = 'build'
                request = self.http.request("GET", self.prisma_policies_url, headers=headers, fields=query_params)  # type:ignore[no-untyped-call]
                self.prisma_policies_response = json.loads(request.data.decode("utf8"))
                logging.debug("Got Prisma build policy metadata")
            else:
                logging.warning("Skipping get prisma build policies. --policy-metadata-filter will not be applied.")
        except Exception:
            logging.warning(f"Failed to get prisma build policy metadata from {self.platform_run_config_url}", exc_info=True)

    def get_prisma_policy_filters(self) -> Dict[str, Dict[str, Any]]:
        try:
            token = self.get_auth_token()
            headers = merge_dicts(get_prisma_auth_header(token), get_prisma_get_headers())

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return {}

            logging.debug(f'Prisma filter URL: {self.prisma_policy_filters_url}')
            request = self.http.request("GET", self.prisma_policy_filters_url, headers=headers)  # type:ignore[no-untyped-call]
            policy_filters: dict[str, dict[str, Any]] = json.loads(request.data.decode("utf8"))
            logging.debug(f'Prisma filter suggestion response: {policy_filters}')
            return policy_filters
        except Exception:
            logging.warning(f"Failed to get prisma build policy metadata from {self.platform_run_config_url}", exc_info=True)
            return {}

    @staticmethod
    def is_valid_policy_filter(policy_filter: dict[str, str], valid_filters: dict[str, dict[str, Any]] | None = None) -> bool:
        """
        Validates only the filter names
        """
        valid_filters = valid_filters or {}

        if not policy_filter:
            return False
        if not valid_filters:
            return False
        for filter_name, filter_value in policy_filter.items():
            if filter_name not in valid_filters.keys():
                logging.warning(f"Invalid filter name: {filter_name}")
                logging.warning(f"Available filter names: {', '.join(valid_filters.keys())}")
                return False
            elif filter_name == 'policy.subtype' and filter_value != 'build':
                logging.warning(f"Filter value not allowed: {filter_value}")
                logging.warning("Available options: build")
                return False
            elif filter_name == 'policy.enabled' and not convert_str_to_bool(filter_value):
                logging.warning(f"Filter value not allowed: {filter_value}")
                logging.warning("Available options: True")
                return False
        logging.debug("--policy-metadata-filter is valid")
        return True

    def get_public_run_config(self) -> None:
        if self.skip_download is True:
            logging.debug("Skipping checkov mapping and guidelines API call")
            return
        try:
            headers: dict[str, Any] = {}

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return

            request = self.http.request("GET", self.guidelines_api_url, headers=headers)  # type:ignore[no-untyped-call]
            if request.status >= 300:
                request = self.http.request("GET", self.guidelines_api_url_backoff, headers=headers)  # type:ignore[no-untyped-call]

            self.public_metadata_response = json.loads(request.data.decode("utf8"))
            platform_type = PRISMA_PLATFORM if self.is_prisma_integration() else BRIDGECREW_PLATFORM
            logging.debug(f"Got checkov mappings and guidelines from {platform_type} platform")
        except Exception:
            logging.warning(f"Failed to get the checkov mappings and guidelines from {self.guidelines_api_url}. Skips using BC_* IDs will not work.",
                            exc_info=True)

    def onboarding(self) -> None:
        if not self.bc_api_key:
            print(Style.BRIGHT + colored(
                "\nWould you like to “level up” your Checkov powers for free?  The upgrade includes: \n\n", 'green',
                attrs=['bold']) + colored(
                u"\u2022 " + "Command line docker Image scanning\n"
                             u"\u2022 " + "Software Composition Analysis\n"
                                          u"\u2022 " + "Centralized policy management\n"
                                                       u"\u2022 " + "Free bridgecrew.cloud account with API access\n"
                                                                    u"\u2022 " + "Auto-fix remediation suggestions\n"
                                                                                 u"\u2022 " + "Enabling of VS Code Plugin\n"
                                                                                              u"\u2022 " + "Dashboard visualisation of Checkov scans\n"
                                                                                                           u"\u2022 " + "Integration with GitHub for:\n"
                                                                                                                        "\t" + u"\u25E6 " + "\tAutomated Pull Request scanning\n"
                                                                                                                                            "\t" + u"\u25E6 " + "\tAuto remediation PR generation\n"
                                                                                                                                                                u"\u2022 " + "Integration with up to 100 cloud resources for:\n"
                                                                                                                                                                             "\t" + u"\u25E6 " + "\tAutomated cloud resource checks\n"
                                                                                                                                                                                                 "\t" + u"\u25E6 " + "\tResource drift detection\n"
                                                                                                                                                                                                                     "\n"
                                                                                                                                                                                                                     "\n" + "and much more...",
                'yellow') +
                colored("\n\nIt's easy and only takes 2 minutes. We can do it right now!\n\n"
                        "To Level-up, press 'y'... \n",
                        'cyan') + Style.RESET_ALL)
            reply = self._input_levelup_results()
            if reply[:1] == 'y':
                print(Style.BRIGHT + colored("\nOk, let’s get you started on creating your free account! \n"
                                             "\nEnter your email address to begin: ", 'green',
                                             attrs=['bold']) + colored(
                    " // This will be used as your login at https://bridgecrew.cloud.\n", 'green'))
                if not self.bc_api_key:
                    email = self._input_email()
                    print(Style.BRIGHT + colored("\nLooks good!"
                                                 "\nNow choose an Organisation Name: ", 'green',
                                                 attrs=['bold']) + colored(
                        " // This will enable collaboration with others who you can add to your team.\n", 'green'))
                    org = self._input_orgname()
                    print(Style.BRIGHT + colored("\nAmazing!"
                                                 "\nWe are now generating a personal API key to immediately enable some new features… ",
                                                 'green', attrs=['bold']))

                    bc_api_token, response = self.get_api_token(email, org)
                    self.bc_api_key = bc_api_token
                    if response.status_code == 200:
                        print(Style.BRIGHT + colored("\nComplete!", 'green', attrs=['bold']))
                        print('\nSaving API key to {}'.format(bridgecrew_file))
                        print(Style.BRIGHT + colored(
                            "\nCheckov will automatically check this location for a key.  If you forget it you’ll find it here\nhttps://bridgecrew.cloud/integrations/api-token\n\n",
                            'green'))
                        persist_key(self.bc_api_key)
                        print(Style.BRIGHT + colored(
                            "Checkov Dashboard is configured, opening https://bridgecrew.cloud to explore your new powers.",
                            'green', attrs=['bold']))
                        print(Style.BRIGHT + colored("FYI - check your inbox for login details! \n", 'green'))

                        print(Style.BRIGHT + colored(
                            "Congratulations! You’ve just super-sized your Checkov!  Why not test-drive image scanning now:",
                            'cyan'))

                        print(Style.BRIGHT + colored(
                            "\ncheckov --docker-image ubuntu --dockerfile-path /Users/bob/workspaces/bridgecrew/Dockerfile --repo-id bob/test --branch master\n",
                            'white'))

                        print(Style.BRIGHT + colored(
                            "Or download our VS Code plugin:  https://github.com/bridgecrewio/checkov-vscode \n",
                            'cyan', attrs=['bold']))

                        print(Style.BRIGHT + colored(
                            "Interested in contributing to Checkov as an open source developer.  We thought you’d never ask.  Check us out at: \nhttps://github.com/bridgecrewio/checkov/issues?q=is%3Aopen+is%3Aissue+label%3A%22good+first+issue%22 \n",
                            'white', attrs=['bold']))

                    else:
                        print(
                            Style.BRIGHT + colored("\nCould not create account, please try again on your next scan! \n",
                                                   'red', attrs=['bold']) + Style.RESET_ALL)
                    webbrowser.open(
                        "https://bridgecrew.cloud/?utm_source=cli&utm_medium=organic_oss&utm_campaign=checkov")
            else:
                print(
                    "\n To see the Dashboard prompt again, run `checkov` with no arguments \n For Checkov usage, try `checkov --help`")
        else:
            print("No argument given. Try ` --help` for further information")

    def get_report_to_platform(self, args: argparse.Namespace, scan_reports: list[Report]) -> None:
        if self.bc_api_key:

            if args.directory:
                repo_id = self.get_repository(args)
                self.setup_bridgecrew_credentials(repo_id=repo_id)
            if self.is_integration_configured():
                self._upload_run(args, scan_reports)

    # Added this to generate a default repo_id for cli scans for upload to the platform
    # whilst also persisting a cli repo_id into the object
    def persist_bc_api_key(self, args: argparse.Namespace) -> str | None:
        if args.bc_api_key:
            self.bc_api_key = args.bc_api_key
        else:
            # get the key from file
            self.bc_api_key = read_key()
        return self.bc_api_key

    # Added this to generate a default repo_id for cli scans for upload to the platform
    # whilst also persisting a cli repo_id into the object
    def persist_repo_id(self, args: argparse.Namespace) -> str:
        if args.repo_id is None:
            if CI_METADATA_EXTRACTOR.from_branch:
                self.repo_id = CI_METADATA_EXTRACTOR.from_branch
            if args.directory:
                basename = path.basename(os.path.abspath(args.directory[0]))
                self.repo_id = f"cli_repo/{basename}"
            if args.file:
                # Get the base path of the file based on it's absolute path
                basename = os.path.basename(os.path.dirname(os.path.abspath(args.file[0])))
                self.repo_id = f"cli_repo/{basename}"

        else:
            self.repo_id = args.repo_id

        if not self.repo_id:
            # this should not happen
            self.repo_id = "cli_repo/unknown"

        return self.repo_id

    def get_repository(self, args: argparse.Namespace) -> str:
        if CI_METADATA_EXTRACTOR.from_branch:
            return CI_METADATA_EXTRACTOR.from_branch
        basename = 'unnamed_repo' if path.basename(args.directory[0]) == '.' else path.basename(args.directory[0])
        repo_id = f"cli_repo/{basename}"
        return repo_id

    def get_api_token(self, email: str, org: str) -> tuple[str, Response]:
        response = self._create_bridgecrew_account(email, org)
        bc_api_token = response.json()["checkovSignup"]
        return bc_api_token, response

    def _upload_run(self, args: argparse.Namespace, scan_reports: list[Report]) -> None:
        print(Style.BRIGHT + colored("Connecting to Bridgecrew.cloud...", 'green',
                                     attrs=['bold']) + Style.RESET_ALL)
        self.persist_repository(args.directory[0])
        print(Style.BRIGHT + colored("Metadata upload complete", 'green',
                                     attrs=['bold']) + Style.RESET_ALL)
        self.persist_scan_results(scan_reports)
        print(Style.BRIGHT + colored("Report upload complete", 'green',
                                     attrs=['bold']) + Style.RESET_ALL)
        self.commit_repository(args.branch)
        print(Style.BRIGHT + colored(
            "COMPLETE! \nYour results are in your Bridgecrew dashboard, available here: https://bridgecrew.cloud \n",
            'green', attrs=['bold']) + Style.RESET_ALL)

    def _create_bridgecrew_account(self, email: str, org: str) -> Response:
        """
        Create new bridgecrew account
        :param email: email of account owner
        :return: account creation response
        """
        payload = {
            "owner_email": email,
            "org": org,
            "source": ONBOARDING_SOURCE,
            "customer_name": org
        }
        response = requests.request("POST", self.onboarding_url, headers=SIGNUP_HEADER, json=payload)
        if response.status_code == 200:
            return response
        else:
            raise Exception("failed to create a bridgecrew account. An organization with this name might already "
                            "exist with this email address. Please login bridgecrew.cloud to retrieve access key")

    def _input_orgname(self) -> str:
        while True:
            result = str(input('Organization name: ')).lower().strip()  # nosec
            # remove spaces and special characters
            result = ''.join(e for e in result if e.isalnum())
            if result:
                break
        return result

    def _input_visualize_results(self) -> str:
        while True:
            result = str(input('Visualize results? (y/n): ')).lower().strip()  # nosec
            if result[:1] in ["y", "n"]:
                break
        return result

    def _input_levelup_results(self) -> str:
        while True:
            result = str(input('Level up? (y/n): ')).lower().strip()  # nosec
            if result[:1] in ["y", "n"]:
                break
        return result

    def _input_email(self) -> str:
        while True:
            email = str(input('E-Mail: ')).lower().strip()  # nosec
            if re.search(EMAIL_PATTERN, email):
                break
            else:
                print("email should match the following pattern: {}".format(EMAIL_PATTERN))
        return email

    @staticmethod
    def loading_output(msg: str) -> None:
        with trange(ACCOUNT_CREATION_TIME) as t:
            for _ in t:
                t.set_description(msg)
                t.set_postfix(refresh=False)
                sleep(SLEEP_SECONDS)

    def repo_matches(self, repo_name: str) -> bool:
        # matches xyz_org/repo or org/repo (where xyz is the BC org name and the CLI repo prefix from the platform)
        return re.match(re.compile(f'^(\\w+_)?{self.repo_id}$'), repo_name) is not None

    def get_default_headers(self, request_type: str) -> dict[str, Any]:
        if not self.bc_source:
            logging.warning("Source was not set")
            return {}

        if request_type.upper() == "GET":
            return merge_dicts(get_default_get_headers(self.bc_source, self.bc_source_version),
                               {"Authorization": self.get_auth_token()})
        elif request_type.upper() == "POST":
            return merge_dicts(get_default_post_headers(self.bc_source, self.bc_source_version),
                               {"Authorization": self.get_auth_token()})

        logging.info(f"Unsupported request {request_type}")
        return {}


bc_integration = BcPlatformIntegration()



from enum import Enum


class ValidationStatus(Enum):
    PRIVILEGED = 'Privileged'
    VALID = 'Valid'
    INVALID = 'Invalid'
    UNKNOWN = 'Unknown'
    UNAVAILABLE = 'Unavailable'

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value


SECRET_VALIDATION_STATUSES = [ValidationStatus.VALID.value,
                              ValidationStatus.PRIVILEGED.value,
                              ValidationStatus.INVALID.value,
                              ValidationStatus.UNKNOWN.value,
                              ValidationStatus.UNAVAILABLE.value]


class VerifySecretsResult(Enum):
    INSUFFICIENT_PARAMS = 'INSUFFICIENT_PARAMS'
    FAILURE = 'FAILURE'
    SUCCESS = 'SUCESS'

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value


GIT_HISTORY_NOT_BEEN_REMOVED = 'not-removed'



import re

seconds_per_unit = {"s": 1, "m": 60, "h": 3600, "d": 86400, "w": 604800}
seconds_per_unit_regex = r"^\d+[s|m|h|d|w]"


# method 'str.removeprefix()' was added in Python 3.9
def removeprefix(input_str: str, prefix: str) -> str:
    if input_str.startswith(prefix):
        return input_str[len(prefix):]
    return input_str


def convert_to_seconds(input_str: str) -> int:
    if re.search(seconds_per_unit_regex, input_str) is None:
        raise Exception(f"format error for input str, usage: {seconds_per_unit_regex}")
    return int(input_str[:-1]) * seconds_per_unit[input_str[-1]]



from typing import Optional, Any, Dict

from .length_equals_attribute_solver import LengthEqualsAttributeSolver
from checkov.common.graph.checks_infra.enums import Operators


class LengthNotEqualsAttributeSolver(LengthEqualsAttributeSolver):
    operator = Operators.LENGTH_NOT_EQUALS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        return not super()._get_operation(vertex, attribute)



from __future__ import annotations

import itertools
import logging
from collections import defaultdict
from typing import List, Dict, Any

from colorama import Style
from prettytable import PrettyTable, SINGLE_BORDER
from termcolor import colored

from checkov.common.bridgecrew.severities import BcSeverities
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record, DEFAULT_SEVERITY
from checkov.common.output.common import compare_table_items_severity
from checkov.policies_3d.record import Policy3dRecord

TABLE_WIDTH = 136


def merge_line_with_previous_table(line: str, table: PrettyTable) -> str:
    # hack to make multiple package tables look like one
    line = line.replace(table.top_junction_char, table.junction_char)
    line = line.replace(table.top_left_junction_char, table.left_junction_char)
    line = line.replace(table.top_right_junction_char, table.right_junction_char)
    return line


def create_iac_code_blocks_output(record: Policy3dRecord) -> str:
    suppress_comment = ""
    check_message = colored('Check: {}: "{}"\n'.format(record.get_output_id(use_bc_ids=True), record.check_name),
                            "white")
    guideline_message = Record.get_guideline_string(record.guideline)
    severity_message = f'\tSeverity: {record.severity.name}\n' if record.severity else ''
    resource_blocks = ''
    resource_block_ids = set()

    for iac_record in record.iac_records:
        resource_id = f'{iac_record.file_path}:{iac_record.resource}'
        if resource_id in resource_block_ids:
            # no need to print the same resource block twice
            continue

        resource_details = colored(f'\n\tResource: {iac_record.file_path}:{iac_record.resource}', attrs=['bold'])
        code_lines = Record.get_code_lines_string(iac_record.code_block)
        detail = Record.get_details_string(iac_record.details)
        caller_file_details = Record.get_caller_file_details_string(iac_record.caller_file_path, iac_record.caller_file_line_range)
        evaluation_message = Record.get_evaluation_string(iac_record.evaluations, iac_record.code_block)

        resource_blocks += f'{detail}{caller_file_details}{resource_details}{code_lines}{evaluation_message}'
        resource_block_ids.add(resource_id)

    if record.check_result["result"] == CheckResult.FAILED and resource_blocks:
        return f"{check_message}{severity_message}{guideline_message}{resource_blocks}"

    if record.check_result["result"] == CheckResult.SKIPPED:
        return f"{check_message}{severity_message}{suppress_comment}{guideline_message}"
    else:
        return f"{check_message}{severity_message}{guideline_message}"


def create_cli_output(*records: list[Policy3dRecord]) -> str:
    cli_outputs = []

    for record in itertools.chain(*records):
        iac_code_blocks_output = create_iac_code_blocks_output(record)
        if iac_code_blocks_output:
            cli_outputs.append(iac_code_blocks_output + Style.RESET_ALL)

        iac_violations_table = render_iac_violations_table(record)
        if iac_violations_table:
            cli_outputs.append(iac_violations_table + Style.RESET_ALL)

        cves_table = render_cve_output(record)
        if cves_table:
            cli_outputs.append(cves_table + Style.RESET_ALL)

        secrets_table = ''  # noqa: F841
        # TODO create a function for creating secrets table, when secrets get into 3d policies
        # Table should have the columns: Secret, Secrety Type, Violation ID, Validation Status (value in red)

    return "\n".join(cli_outputs)


def render_cve_output(record: Policy3dRecord) -> str | None:
    if not record.vulnerabilities:
        #  this shouldn't happen
        logging.error(f"'vulnerabilities' is not set for {record.check_id}")
        return None

    package_cves_details_map: dict[str, dict[str, Any]] = defaultdict(dict)

    for cve in record.vulnerabilities:
        image_name = cve.get('dockerImageName')
        package_name = cve.get('packageName', '')
        package_version = cve.get('packageVersion')
        severity_str = cve.get('severity', BcSeverities.NONE).upper()

        package_cves_details_map[package_name].setdefault("cves", []).append(
            {
                "id": cve.get('cveId'),
                "severity": severity_str
            }
        )

        if package_name in package_cves_details_map:
            package_cves_details_map[package_name]["cves"].sort(key=compare_table_items_severity, reverse=True)
            package_cves_details_map[package_name]["current_version"] = package_version
            package_cves_details_map[package_name]["image_name"] = image_name

    if package_cves_details_map:
        return (
            create_cli_cves_table(
                file_path=record.file_path,
                package_details_map=package_cves_details_map,
            )
        )

    return None


def create_cli_cves_table(file_path: str, package_details_map: Dict[str, Dict[str, Any]]) -> str:
    columns = 5
    column_width = int(TABLE_WIDTH / columns)

    package_table_lines = create_package_overview_table_part(
        table_width=TABLE_WIDTH, column_width=column_width, package_details_map=package_details_map
    )

    return (
        Style.BRIGHT +
        f"\tImage Referenced with Matching CVEs:\n"
        f"{''.join(package_table_lines)}\n" +
        Style.RESET_ALL
    )


def create_package_overview_table_part(
        table_width: int, column_width: int, package_details_map: Dict[str, Dict[str, Any]]
) -> List[str]:
    package_table_lines: List[str] = []
    package_table = PrettyTable(
        min_table_width=table_width,
        max_table_width=table_width
    )
    package_table.set_style(SINGLE_BORDER)
    package_table.field_names = [
        "Image",
        "Package",
        "Current version",
        "CVE ID",
        "Severity"
    ]
    for package_idx, (package_name, details) in enumerate(package_details_map.items()):
        if package_idx > 0:
            del package_table_lines[-1]
            package_table.header = False
            package_table.clear_rows()

        for cve_idx, cve in enumerate(details["cves"]):
            col_image = ""
            col_package = ""
            col_current_version = ""
            if cve_idx == 0:
                col_image = details["image_name"]
                col_package = package_name
                col_current_version = details["current_version"]

            package_table.add_row(
                [
                    col_image,
                    col_package,
                    col_current_version,
                    cve["id"],
                    cve["severity"]
                ]
            )

        package_table.align = "l"
        package_table.min_width = column_width
        package_table.max_width = column_width

        for line in package_table.get_string().splitlines(keepends=True):
            if package_idx > 0:
                # hack to make multiple package tables look like one
                line = merge_line_with_previous_table(line, package_table)
            package_table_lines.append(f"\t{line}")

    return package_table_lines


def render_iac_violations_table(record: Policy3dRecord) -> str | None:
    if not record.iac_records:
        #  this shouldn't happen
        logging.error(f"'iac_records' is not set for {record.check_id}")
        return None

    resource_violation_details_map: dict[str, dict[str, Any]] = defaultdict(dict)

    for iac_record in record.iac_records:
        resource = iac_record.resource
        severity = (iac_record.severity.name if iac_record.severity else DEFAULT_SEVERITY).upper()
        resource_violation_details_map[resource].setdefault('violations', []).append(
            {
                'id': iac_record.bc_check_id,
                'title': iac_record.check_name,
                'severity': severity
            }
        )

        if resource in resource_violation_details_map:
            resource_violation_details_map[resource]['violations'].sort(key=compare_table_items_severity, reverse=True)

    if len(resource_violation_details_map.keys()):
        return (
            create_iac_violations_table(
                file_path=record.file_path,
                resource_violation_details_map=resource_violation_details_map,
            )
        )

    return None


def create_iac_violations_table(file_path: str, resource_violation_details_map: Dict[str, Dict[str, Any]]) -> str:
    columns = 5  # it really has only 4 columns, but the title would get a width of two columns
    column_width = int(TABLE_WIDTH / columns)

    iac_table_lines = create_iac_violations_overview_table_part(
        table_width=TABLE_WIDTH, column_width=column_width, resource_violation_details_map=resource_violation_details_map
    )

    return (
        Style.BRIGHT +
        f"\tMatching IaC violations:\n"
        f"{''.join(iac_table_lines)}\n" +
        Style.RESET_ALL
    )


def create_iac_violations_overview_table_part(
        table_width: int, column_width: int, resource_violation_details_map: Dict[str, Dict[str, Any]]
) -> List[str]:
    iac_table_lines: List[str] = []
    iac_table = PrettyTable(
        min_table_width=table_width,
        max_table_width=table_width
    )
    iac_table.set_style(SINGLE_BORDER)
    iac_table.field_names = [
        "Resource",
        "Violation",
        "Title",
        "Severity"
    ]
    for resource_idx, (resource, details) in enumerate(resource_violation_details_map.items()):
        if resource_idx > 0:
            del iac_table_lines[-1]
            iac_table.header = False
            iac_table.clear_rows()

        for violation_idx, violation in enumerate(details['violations']):
            col_resource = ''
            if violation_idx == 0:
                col_resource = resource

            iac_table.add_row(
                [
                    col_resource,
                    violation['id'],
                    violation['title'],
                    violation['severity']
                ]
            )

        iac_table.align = "l"
        # the column widths are manipulated here with -2s so all the printed tables have eventually the same width
        regular_width = column_width - 2
        double_width = 2 * column_width - 2

        iac_table.min_width = regular_width
        iac_table.max_width = regular_width
        iac_table.min_width['Title'] = double_width  # type:ignore[index]
        iac_table.max_width['Title'] = double_width  # type:ignore[index]

        for line in iac_table.get_string().splitlines(keepends=True):
            if resource_idx > 0:
                line = merge_line_with_previous_table(line, iac_table)
            iac_table_lines.append(f"\t{line}")

    return iac_table_lines



from typing import List, Dict, Any, Tuple, Optional

from checkov.common.bridgecrew.severities import Severity
from checkov.common.output.record import Record
from checkov.common.typing import _CheckResult


class Policy3dRecord(Record):
    def __init__(self,
                 check_id: str,
                 bc_check_id: str,
                 check_name: str,
                 check_result: _CheckResult,
                 code_block: List[Tuple[int, str]],
                 file_path: str,
                 file_line_range: List[int],
                 resource: str,
                 evaluations: Optional[Dict[str, Any]],
                 check_class: str,
                 file_abs_path: str,
                 severity: Optional[Severity],
                 vulnerabilities: List[Dict[str, Any]],
                 iac_records: List[Record]
                 ) -> None:
        super().__init__(
            check_id=check_id,
            bc_check_id=bc_check_id,
            check_name=check_name,
            check_result=check_result,
            code_block=code_block,
            file_path=file_path,
            file_line_range=file_line_range,
            resource=resource,
            evaluations=evaluations,
            check_class=check_class,
            file_abs_path=file_abs_path,
            severity=severity,
        )
        self.vulnerabilities = vulnerabilities
        self.iac_records = iac_records



from __future__ import annotations

import logging
from abc import abstractmethod
from pathlib import Path
from typing import Any

from checkov.common.graph.graph_builder import Edge
from checkov.common.graph.graph_builder.graph_components.blocks import Block
from checkov.common.graph.graph_builder.local_graph import LocalGraph


class ObjectLocalGraph(LocalGraph[Block]):
    def __init__(self, definitions: dict[str | Path, dict[str, Any] | list[dict[str, Any]]]) -> None:
        super().__init__()
        self.vertices: list[Block] = []
        self.definitions = definitions
        self.vertices_by_path_and_name: dict[tuple[str, str], int] = {}

    def build_graph(self, render_variables: bool = False) -> None:
        self._create_vertices()
        logging.debug(f"[{self.__class__.__name__}] created {len(self.vertices)} vertices")

        for i, vertex in enumerate(self.vertices):
            self.vertices_by_block_type[vertex.block_type].append(i)
            self.vertices_block_name_map[vertex.block_type][vertex.name].append(i)
            self.vertices_by_path_and_name[(vertex.path, vertex.name)] = i

            self.in_edges[i] = []
            self.out_edges[i] = []

        self._create_edges()
        logging.debug(f"[{self.__class__.__name__}] created {len(self.edges)} edges")

    @abstractmethod
    def _create_vertices(self) -> None:
        pass

    @abstractmethod
    def _create_edges(self) -> None:
        pass

    def _create_edge(self, origin_vertex_index: int, dest_vertex_index: int, label: str = "default") -> None:
        if origin_vertex_index == dest_vertex_index:
            # this should not happen
            return

        edge = Edge(origin_vertex_index, dest_vertex_index, label)
        self.edges.append(edge)
        self.out_edges[origin_vertex_index].append(edge)
        self.in_edges[dest_vertex_index].append(edge)

    @staticmethod
    @abstractmethod
    def get_files_definitions(root_folder: str | Path) -> dict[str | Path, dict[str, Any] | list[dict[str, Any]]]:
        """This is temporary till I have a better idea"""
        pass
